@@ -57,11 +57,11 @@ static void dismiss_alarm( e_snooze snooze ){
 	int status;
 	if(snooze){
 		// snooze
-		highlight_selected_area(0, window.height-59, (window.width>>1)-1,59);
+		highlight_selected_area(0, window.height-59, (window.width>>1)-1,59, 0);
 		status = ALARM_DIALOG_RESPONSE_SNOOZE;
 	} else {
 		// dismiss
-		highlight_selected_area((window.width>>1)+1, window.height-59, (window.width>>1)-1,59);
+		highlight_selected_area((window.width>>1)+1, window.height-59, (window.width>>1)-1,59, 0);
 		status = ALARM_DIALOG_RESPONSE_DISMISS;
 	}
 	#ifdef ARCH_armel
@@ -90,17 +90,17 @@ void draw_alarm( unsigned force ){
 	x_set_color(bg_color);
 	x_fill_rectangle(0, 0, window.width, window.height);
 
-	draw_text("Alarm", 64, red_status_color, 20, 20, ALIGN_LEFT, ALIGN_TOP, 0);
+	draw_text("Alarm", 64, red_color, 20, 20, ALIGN_LEFT, ALIGN_TOP, 0);
 
 	// timestamp
 	struct tm *tm = localtime(&alarm_timestamp);
 	char buffer[20];
 	strftime(buffer, sizeof(buffer), (cfg_clock_24h ? "%H:%M" : (cfg_clock_ampm ? "%l:%M %P" : "%l:%M")), tm); // time of day
-	draw_text((*buffer == ' ' ? &buffer[1] : buffer), 64, red_status_color, window.width-20, 10, ALIGN_RIGHT, ALIGN_TOP, 0);
+	draw_text((*buffer == ' ' ? &buffer[1] : buffer), 64, red_color, window.width-20, 10, ALIGN_RIGHT, ALIGN_TOP, 0);
 	strftime(buffer, sizeof(buffer), "%A", tm); // long day of week
-	draw_text(buffer, 36, red_status_color, window.width-20, 65, ALIGN_RIGHT, ALIGN_TOP, 0);
+	draw_text(buffer, 36, red_color, window.width-20, 65, ALIGN_RIGHT, ALIGN_TOP, 0);
 	strftime(buffer, sizeof(buffer), "%e %B", tm); // day and long month
-	draw_text((*buffer == ' ' ? &buffer[1] : buffer), 36, red_status_color, window.width-20, 105, ALIGN_RIGHT, ALIGN_TOP, 0);
+	draw_text((*buffer == ' ' ? &buffer[1] : buffer), 36, red_color, window.width-20, 105, ALIGN_RIGHT, ALIGN_TOP, 0);
 
 	// description
 	unsigned w, h, lines;
@@ -110,7 +110,7 @@ void draw_alarm( unsigned force ){
 	if(lines > h) lines = h;
 	draw_description_line(alarm_description, 0, w, h, 145+((window.height-205)>>1)-((lines*DESC_LINE_HEIGHT)>>1));
 
-	x_set_color(line_color);
+	x_set_color(line1_color);
 	x_draw_line(0, window.height-60, window.width, window.height-60);
 //	if(use_minimal_ui){
 //		// power button message
@@ -118,9 +118,9 @@ void draw_alarm( unsigned force ){
 //	} else {
 		// close and snooze buttons
 		x_draw_line(window.width>>1, window.height-60, window.width>>1, window.height);
-		draw_text("Close", 24, fg2_color, window.width - (window.width>>2), window.height-20, ALIGN_CENTER, ALIGN_BASELINE, 0);
+		draw_text("Close", 24, button_color, window.width - (window.width>>2), window.height-20, ALIGN_CENTER, ALIGN_BASELINE, 0);
 		if(alarm_mode == ALARM_MODE_NORMAL)
-			draw_text("Snooze", 24, fg2_color, window.width>>2, window.height-20, ALIGN_CENTER, ALIGN_BASELINE, 0);
+			draw_text("Snooze", 24, button_color, window.width>>2, window.height-20, ALIGN_CENTER, ALIGN_BASELINE, 0);
 //	}
 }
 
