@@ -13,7 +13,7 @@ void draw_edit_widgets_button( unsigned force, int x, int y ){
 		// background for all buttons is cleared by draw_window()
 //	} else if(/* condition */){
 //		// button background
-//		x_set_color(bg_color);
+//		x_set_color(secondary_bg_color);
 //		x_fill_rectangle(x, y, 80, 80);
 	} else return;
 
@@ -30,7 +30,7 @@ void draw_edit_widgets_button( unsigned force, int x, int y ){
 void click_edit_widgets_button( ){
 	if(click.type != CLICK_SHORT) return;
 
-	highlight_selected_area(widget_x, widget_y, 80,80);
+	highlight_selected_area(widget_x, widget_y, 80,80, 1);
 	widget_edit_mode = 1;
 	selected_cell.widget_cell = NULL;
 
