@@ -27,6 +27,7 @@ const char *conf_screen_autolock2 = "/system/osso/dsm/locks/touchscreen_keypad_a
 const char *conf_screen_stayslit1 = "/apps/osso/applet/osso-applet-display/display_charger";
 const char *conf_screen_stayslit2 = "/system/osso/dsm/display/display_on_with_charger";
 
+const char *conf_ignore_master_volume = "/apps/asui_state/ignore_master_volume";
 const char *conf_brightness_level = "/apps/asui_state/brightness_level";
 const char *conf_theme = "/apps/asui_state/theme";
 const char *conf_ssh_server = "/apps/asui_state/ssh_server";
@@ -48,6 +49,9 @@ const char *conf_alarm_timestamp = "/apps/asui_state/alarm_timestamp";
 
 GConfClient *gconf_client;
 
+// statusbar
+unsigned cfg_statusbar_battery_enable, cfg_statusbar_wifi_enable, cfg_statusbar_bluetooth_enable, cfg_statusbar_drain_enable, cfg_statusbar_cpu_enable;
+
 // battery
 unsigned cfg_battery_idle_time, cfg_memory_format, cfg_swap_format, cfg_temperature_F, cfg_history_days, cfg_track_usage_events;
 // clock
@@ -55,6 +59,8 @@ unsigned cfg_clock_24h, cfg_clock_ampm;
 unsigned cfg_popup_duration, cfg_popup_onmap, cfg_popup_fullscreen, cfg_popup_cooldown, cfg_popup_noasui;
 // brightness
 unsigned cfg_brightness_enhanced, cfg_brightness_allow_zero;
+// WiFi
+unsigned cfg_wifi_disconnect_fix;
 // BRV keys
 unsigned cfg_buttons_rotate, cfg_buttons_brightness, cfg_buttons_brightness_step, cfg_buttons_volume, cfg_buttons_volume_step;
 // button modes
@@ -67,6 +73,8 @@ unsigned cfg_power_action_single, cfg_power_action_double, cfg_power_longpress_s
 unsigned cfg_secure_button, cfg_secure_action, cfg_secure_power_action_single, cfg_secure_power_action_double;
 // user interface
 unsigned cfg_tkactive_notifications, cfg_service_size_unit, cfg_short_tap_markers, cfg_start_visible, cfg_threshold_black, cfg_threshold_white;
+char *cfg_theme_file_black = NULL, *cfg_theme_file_white = NULL;
+unsigned cfg_window_slide_unmap, cfg_window_slide_map;
 // alarm
 unsigned cfg_alarm_duration, cfg_alarm_volume_min, cfg_alarm_volume_max, cfg_alarm_volume_step;
 
@@ -115,6 +123,13 @@ s_button *button_cells[MAX_BUTTON_CELLS];
 
 #ifdef ARCH_armel
 
+// statusbar
+static const char *conf_statusbar_battery_enable = "/apps/asui/statusbar_battery_enable";
+static const char *conf_statusbar_wifi_enable = "/apps/asui/statusbar_wifi_enable";
+static const char *conf_statusbar_bluetooth_enable = "/apps/asui/statusbar_bluetooth_enable";
+static const char *conf_statusbar_drain_enable = "/apps/asui/statusbar_drain_enable";
+static const char *conf_statusbar_cpu_enable = "/apps/asui/statusbar_cpu_enable";
+
 // battery
 static const char *conf_battery_idle_time = "/apps/asui/battery_idle_time";
 static const char *conf_memory_format = "/apps/asui/memory_format";
@@ -122,7 +137,6 @@ static const char *conf_swap_format = "/apps/asui/swap_format";
 static const char *conf_temperature_F = "/apps/asui/temperature_F";
 static const char *conf_history_days = "/apps/asui/history_days";
 static const char *conf_track_usage_events = "/apps/asui/track_usage_events";
-static const char *conf_enable_battery_applet = "/apps/asui/enable_battery_applet"; // not used by ASUI
 // clock
 static const char *conf_clock_24h = "/apps/asui/clock_24h";
 static const char *conf_clock_ampm = "/apps/asui/clock_ampm";
@@ -134,6 +148,8 @@ static const char *conf_popup_noasui = "/apps/asui/popup_noasui";
 // brightness
 static const char *conf_brightness_enhanced = "/apps/asui/brightness_enhanced";
 static const char *conf_brightness_allow_zero = "/apps/asui/brightness_allow_zero";
+// WiFi
+static const char *conf_wifi_disconnect_fix = "/apps/asui/wifi_disconnect_fix";
 // BRV keys
 static const char *conf_buttons_rotate = "/apps/asui/buttons_rotate";
 static const char *conf_buttons_brightness = "/apps/asui/buttons_brightness";
@@ -170,6 +186,10 @@ static const char *conf_short_tap_markers = "/apps/asui/short_tap_markers";
 static const char *conf_start_visible = "/apps/asui/start_visible";
 static const char *conf_threshold_black = "/apps/asui/threshold_black";
 static const char *conf_threshold_white = "/apps/asui/threshold_white";
+static const char *conf_theme_file_black = "/apps/asui/theme_file_black";
+static const char *conf_theme_file_white = "/apps/asui/theme_file_white";
+static const char *conf_window_slide_unmap = "/apps/asui/window_slide_unmap";
+static const char *conf_window_slide_map = "/apps/asui/window_slide_map";
 // alarm
 static const char *conf_alarm_duration = "/apps/asui/alarm_duration";
 static const char *conf_alarm_volume_min = "/apps/asui/alarm_volume_min";
@@ -240,6 +260,33 @@ static void make_float( float *variable, const char *key, float initial_value, u
 }
 */
 
+#ifdef ARCH_armel
+static void make_string( char **variable, const char *key, char *initial_value, unsigned flag ){
+	GError *error = NULL; GConfError code;
+	char *value;
+
+	// test if key exists by querying with wrong type
+	gconf_client_get_bool(gconf_client, key, &error);
+	if(error == NULL) code = GCONF_ERROR_BAD_KEY; else { code = error->code; g_error_free(error); }
+
+	if(code == GCONF_ERROR_TYPE_MISMATCH){
+		// key exists
+		char *v = gconf_client_get_string(gconf_client, key, NULL);
+		if(v){
+			value = malloc(strlen(v)+1);
+			strcpy(value, v);
+		} else value = "";
+	} else {
+		// create the key
+		if(initial_value) value = initial_value;
+		else value = "";
+		conf_set_string(key, value);
+	}
+	if(*variable == NULL) *variable = "";
+	if(strcmp(*variable, value)){ if(*variable) free(*variable); *variable = value; hw_changes |= flag; }
+}
+#endif
+
 ////////////////////////////////////////////////////////////////////////// BUTTON CELLS
 
 static s_button *button_queue[MAX_BUTTON_CELLS];
@@ -402,6 +449,12 @@ void config_init( ){
 	prev_brightness_enhanced = prev_battery_idle_time = 0;
 
 	#ifdef CLEAR_KEYS
+	// statusbar
+	gconf_client_unset(gconf_client, conf_statusbar_battery_enable, NULL);
+	gconf_client_unset(gconf_client, conf_statusbar_wifi_enable, NULL);
+	gconf_client_unset(gconf_client, conf_statusbar_bluetooth_enable, NULL);
+	gconf_client_unset(gconf_client, conf_statusbar_drain_enable, NULL);
+	gconf_client_unset(gconf_client, conf_statusbar_cpu_enable, NULL);
 	// battery
 	gconf_client_unset(gconf_client, conf_battery_idle_time, NULL);
 	gconf_client_unset(gconf_client, conf_memory_format, NULL);
@@ -409,7 +462,6 @@ void config_init( ){
 	gconf_client_unset(gconf_client, conf_temperature_F, NULL);
 	gconf_client_unset(gconf_client, conf_history_days, NULL);
 	gconf_client_unset(gconf_client, conf_track_usage_events, NULL);
-gconf_client_unset(gconf_client, conf_enable_battery_applet, NULL);
 	// clock
 	gconf_client_unset(gconf_client, conf_clock_24h, NULL);
 	gconf_client_unset(gconf_client, conf_clock_ampm, NULL);
@@ -421,6 +473,8 @@ gconf_client_unset(gconf_client, conf_enable_battery_applet, NULL);
 	// brightness
 	gconf_client_unset(gconf_client, conf_brightness_enhanced, NULL);
 	gconf_client_unset(gconf_client, conf_brightness_allow_zero, NULL);
+	// WiFi
+	gconf_client_unset(gconf_client, conf_wifi_disconnect_fix, NULL);
 	// BRV keys
 	gconf_client_unset(gconf_client, conf_buttons_rotate, NULL);
 	gconf_client_unset(gconf_client, conf_buttons_brightness, NULL);
@@ -457,6 +511,10 @@ gconf_client_unset(gconf_client, conf_enable_battery_applet, NULL);
 	gconf_client_unset(gconf_client, conf_start_visible, NULL);
 	gconf_client_unset(gconf_client, conf_threshold_black, NULL);
 	gconf_client_unset(gconf_client, conf_threshold_white, NULL);
+	gconf_client_unset(gconf_client, conf_theme_file_black, NULL);
+	gconf_client_unset(gconf_client, conf_theme_file_white, NULL);
+	gconf_client_unset(gconf_client, conf_window_slide_unmap, NULL);
+	gconf_client_unset(gconf_client, conf_window_slide_map, NULL);
 
 	gconf_client_unset(gconf_client, conf_alarm_duration, NULL);
 	gconf_client_unset(gconf_client, conf_alarm_volume_min, NULL);
@@ -494,6 +552,7 @@ gconf_client_unset(gconf_client, conf_enable_battery_applet, NULL);
 	gconf_client_unset(gconf_client, widget_ssh.conf, NULL);
 	gconf_client_unset(gconf_client, widget_vpn.conf, NULL);
 	gconf_client_unset(gconf_client, widget_usb.conf, NULL);
+	gconf_client_unset(gconf_client, conf_ignore_master_volume, NULL);
 	gconf_client_unset(gconf_client, conf_brightness_level, NULL);
 	gconf_client_unset(gconf_client, conf_theme, NULL);
 	gconf_client_unset(gconf_client, conf_ssh_server, NULL);
@@ -524,6 +583,11 @@ gconf_client_unset(gconf_client, conf_enable_battery_applet, NULL);
 	gconf_client_unset(gconf_client, "/apps/asui/softpoweroff_mode", NULL); // added and removed in 0.5.7 test releases
 	gconf_client_unset(gconf_client, "/apps/asui/power_action", NULL); // removed in 0.5.8
 	gconf_client_unset(gconf_client, "/apps/asui/secure_power_action", NULL); // removed in 0.5.8
+	gconf_client_unset(gconf_client, "/apps/asui/enable_battery_applet", NULL); // removed in 0.5.9
+	gconf_client_unset(gconf_client, "/apps/asui/enable_battery_applet_wifi", NULL); // added and removed in 0.5.9 test releases
+	gconf_client_unset(gconf_client, "/apps/asui/enable_battery_applet_bluetooth", NULL); // added and removed in 0.5.9 test releases
+	gconf_client_unset(gconf_client, "/apps/asui/enable_battery_applet_drain", NULL); // added and removed in 0.5.9 test releases
+	gconf_client_unset(gconf_client, "/apps/asui/enable_battery_applet_cpu", NULL); // added and removed in 0.5.9 test releases
 
 	first_config_load = 1;
 	config_reload();
@@ -584,6 +648,12 @@ static void free_usage_events( ){
 
 void config_reload( ){
 	#ifdef ARCH_armel
+	// statusbar
+	make_bool(&cfg_statusbar_battery_enable,	conf_statusbar_battery_enable,		1,	0);				// default: 1	enabled
+	make_bool(&cfg_statusbar_wifi_enable,		conf_statusbar_wifi_enable,			1,	0);				// default: 1	enabled
+	make_bool(&cfg_statusbar_bluetooth_enable,	conf_statusbar_bluetooth_enable,	1,	0);				// default: 1	enabled
+	make_bool(&cfg_statusbar_drain_enable,		conf_statusbar_drain_enable,		1,	0);				// default: 1	enabled
+	make_bool(&cfg_statusbar_cpu_enable,		conf_statusbar_cpu_enable,			1,	0);				// default: 1	enabled
 	// battery
 	make_bool(&cfg_battery_idle_time,		conf_battery_idle_time,			0,		HW_BATTERY);		// default: 0	don't show idle time
 	make_uint(&cfg_memory_format,			conf_memory_format,				3,		HW_BATTERY);		// default: 3	used/total
@@ -591,7 +661,6 @@ void config_reload( ){
 	make_bool(&cfg_temperature_F,			conf_temperature_F,				0,		HW_TEMPERATURE);	// default: 0	celsius
 	make_uint(&cfg_history_days,			conf_history_days,				7,		0);					// default: 7	7 days
 	make_bool(&cfg_track_usage_events, 		conf_track_usage_events,		0,		0);					// default: 0	off
-	(void)conf_enable_battery_applet;
 	// clock
 	make_bool(&cfg_clock_24h,				conf_clock_24h,					1,		0);					// default: 1	24 hour clock
 	make_bool(&cfg_clock_ampm,				conf_clock_ampm,				0,		0);					// default: 0	24 hour clock doesn't use this
@@ -603,6 +672,8 @@ void config_reload( ){
 	// brightness
 	make_bool(&cfg_brightness_enhanced,		conf_brightness_enhanced,		!is_770,HW_BRIGHTNESS);		// default: 1	all 127 brightness levels
 	make_bool(&cfg_brightness_allow_zero,	conf_brightness_allow_zero,		0,		HW_BRIGHTNESS);		// default: 0	don't allow zero brightness
+	// WiFi
+	make_bool(&cfg_wifi_disconnect_fix,		conf_wifi_disconnect_fix,		0,		0);					// default: 0	disabled
 	// BRV keys
 	make_uint(&cfg_buttons_rotate,			conf_buttons_rotate,			1,		0);					// default: 1	zoom buttons
 	make_uint(&cfg_buttons_brightness,		conf_buttons_brightness,		2,		0);					// default: 2	up/down buttons
@@ -639,13 +710,18 @@ void config_reload( ){
 	make_bool(&cfg_start_visible,			conf_start_visible,				0,		0);					// default: 0	not visible
 	make_uint(&cfg_threshold_black,			conf_threshold_black,			3000,	HW_THEME);			// default: 	to black theme when below 3000 lux
 	make_uint(&cfg_threshold_white,			conf_threshold_white,			4000,	HW_THEME);			// default: 	to white theme when 4000 lux or greater
+	make_string(&cfg_theme_file_black,		conf_theme_file_black,			"default.bw",HW_THEME_FILE);// default: -	default theme
+	make_string(&cfg_theme_file_white,		conf_theme_file_white,			"default.bw",HW_THEME_FILE);// default: -	default theme
+	make_bool(&cfg_window_slide_unmap,		conf_window_slide_unmap,		1,		0);					// default: 1	enabled
+	make_bool(&cfg_window_slide_map,		conf_window_slide_map,			1,		0);					// default: 1	enabled
 	// alarm
 	make_uint(&cfg_alarm_duration,			conf_alarm_duration,			60,		0);					// default:		20-3600 seconds
 	make_uint(&cfg_alarm_volume_min,		conf_alarm_volume_min,			50,		0);					// default:		10-100 / 100
 	make_uint(&cfg_alarm_volume_max,		conf_alarm_volume_max,			100,	0);					// default:		10-100 / 100
 	make_uint(&cfg_alarm_volume_step,		conf_alarm_volume_step,			125,	0);					// default:		0-1000 / 1000
 	// state
-	unsigned brightness_level;
+	unsigned ignore_master_volume, brightness_level;
+	make_bool(&ignore_master_volume,		conf_ignore_master_volume, 		0,		0);					// default: 0	don't ignore
 	make_uint(&brightness_level,			conf_brightness_level,			31,		HW_BRIGHTNESS);		// default: 31	25% brightness
 	unsigned theme = !is_810;
 	make_uint(&cfg_theme,					conf_theme,						theme,	HW_THEME);			// default: 0/1	color theme
@@ -759,6 +835,10 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 			hw_query_battery(1); // flush
 	}
 
+	// default theme
+	if(*cfg_theme_file_black == '\0'){ set_string(&cfg_theme_file_black, "default.bw", 0); conf_set_string(conf_theme_file_black, cfg_theme_file_black); }
+	if(*cfg_theme_file_white == '\0'){ set_string(&cfg_theme_file_white, "default.bw", 0); conf_set_string(conf_theme_file_white, cfg_theme_file_white); }
+
 	// alarm
 	if(cfg_alarm_duration > 3600 || cfg_alarm_duration < 20){ cfg_alarm_duration = 60; conf_set_int(conf_alarm_duration, 60); }
 	if(cfg_alarm_volume_min > 100 || cfg_alarm_volume_min < 10){ cfg_alarm_volume_min = 50; conf_set_int(conf_alarm_volume_min, 50); }
@@ -847,6 +927,9 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 	cfg_start_visible = 1; // no power button to map it
 	cfg_threshold_white = 4000; // must be set otherwise white theme is used
 
+	set_string(&cfg_theme_file_black, "default.bw", 0);
+	set_string(&cfg_theme_file_white, "default.bw", 0);
+
 	cfg_theme = 1; // no ALS sensor, use blank theme
 	cfg_ssh_server = 0; // openssh
 	cfg_battery_usage = 0; // show active/idle times
@@ -854,9 +937,15 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 
 	#endif
 
-// TODO: control panel applet will send message to main loop when settings need to be reloaded
-
 	debug_write("SETTINGS");
+	// statusbar
+	#ifdef ARCH_armel
+	debug_write("  cfg_statusbar_battery_enable = %u", cfg_statusbar_battery_enable);
+	debug_write("  cfg_statusbar_wifi_enable = %u", cfg_statusbar_wifi_enable);
+	debug_write("  cfg_statusbar_bluetooth_enable = %u", cfg_statusbar_bluetooth_enable);
+	debug_write("  cfg_statusbar_drain_enable = %u", cfg_statusbar_drain_enable);
+	debug_write("  cfg_statusbar_cpu_enable = %u", cfg_statusbar_cpu_enable);
+	#endif
 	// battery
 	debug_write("  cfg_battery_idle_time = %u", cfg_battery_idle_time);
 	debug_write("  cfg_memory_format = %u", cfg_memory_format);
@@ -864,7 +953,6 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 	debug_write("  cfg_temperature_F = %u", cfg_temperature_F);
 	debug_write("  cfg_history_days = %u", cfg_history_days);
 	debug_write("  cfg_track_usage_events = %u", cfg_track_usage_events);
-	debug_write("  cfg_enable_battery_applet = %u", conf_get_bool(conf_enable_battery_applet));
 	// clock
 	debug_write("  cfg_clock_24h = %u", cfg_clock_24h);
 	debug_write("  cfg_clock_ampm = %u", cfg_clock_ampm);
@@ -876,6 +964,8 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 	// brightness
 	debug_write("  cfg_brightness_enhanced = %u", cfg_brightness_enhanced);
 	debug_write("  cfg_brightness_allow_zero = %u", cfg_brightness_allow_zero);
+	// WiFi
+	debug_write("  cfg_wifi_disconnect_fix = %u", cfg_wifi_disconnect_fix);
 	// BRV keys
 	debug_write("  cfg_buttons_rotate = %u", cfg_buttons_rotate);
 	debug_write("  cfg_buttons_brightness = %u", cfg_buttons_brightness);
@@ -912,6 +1002,10 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 	debug_write("  cfg_start_visible = %u", cfg_start_visible);
 	debug_write("  cfg_threshold_black = %u", cfg_threshold_black);
 	debug_write("  cfg_threshold_white = %u", cfg_threshold_white);
+	debug_write("  cfg_theme_file_black = \"%s\"", cfg_theme_file_black);
+	debug_write("  cfg_theme_file_white = \"%s\"", cfg_theme_file_white);
+	debug_write("  cfg_window_slide_unmap = \"%u\"", cfg_window_slide_unmap);
+	debug_write("  cfg_window_slide_map = \"%u\"", cfg_window_slide_map);
 	// alarm
 	debug_write("  cfg_alarm_duration = %u", cfg_alarm_duration);
 	debug_write("  cfg_alarm_volume_min = %u", cfg_alarm_volume_min);
@@ -920,7 +1014,10 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 
 	debug_write("STATE");
 	// state
+	#ifdef ARCH_armel
+	debug_write("  ignore_master_volume = %u", ignore_master_volume);
 	debug_write("  brightness_level = %u", brightness_level);
+	#endif
 	debug_write("  cfg_theme = %u", cfg_theme);
 	debug_write("  cfg_ssh_server = %u", cfg_ssh_server);
 	debug_write("  cfg_battery_usage = %u", cfg_battery_usage);
