@@ -131,6 +131,9 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 
 		config_reload();
 
+		if(hw_changes & (HW_THEME_FILE|HW_THEME))
+			dbus_send_wakeup(_WAKEUP_REFRESH_);
+
 		return DBUS_HANDLER_RESULT_HANDLED;
 
 	} else if(dbus_message_is_method_call(message, "com.qzx.asui", "show")){
@@ -172,6 +175,9 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 		if(!(hw_initialized & HWIS_BME_CAPACITY)){
 			dbus_send_charging_status_signal();
 			dbus_send_battery_capacity_signal();
+			dbus_send_wifi_status_signal();
+			dbus_send_bluetooth_status_signal();
+			dbus_send_display_status_signal();
 		}
 
 		return DBUS_HANDLER_RESULT_HANDLED;
@@ -673,6 +679,12 @@ acting_dead_state = 0;
 			if(hw_flight_mode != flight){
 				hw_flight_mode = flight;
 				if(is_value_visible(flightmode_button_page)){ hw_changes |= HW_FLIGHT_MODE; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+
+				// return to normal mode
+				if(flight && hw_wifi.disconnect_unfix == 1){ usleep(250000); hw_wifi.disconnect_unfix = 0; mce_normal_mode(); }
+
+				dbus_send_wifi_status_signal();
+				dbus_send_bluetooth_status_signal();
 			}
 		}
 
@@ -793,6 +805,8 @@ acting_dead_state = 0;
 				hw_bluetooth.enabled = enabled;
 				hw_bluetooth.visible = visible;
 				if(is_value_visible(bluetooth_widget_page)){ hw_changes |= HW_BLUETOOTH; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+
+				dbus_send_bluetooth_status_signal();
 			}
 		}
 
@@ -819,6 +833,8 @@ acting_dead_state = 0;
 		hw_bluetooth.nr_devices++;
 		if(is_value_visible(bluetooth_widget_page)){ hw_changes |= HW_BLUETOOTH; dbus_send_wakeup(_WAKEUP_REFRESH_); }
 
+		dbus_send_bluetooth_status_signal();
+
 		return DBUS_HANDLER_RESULT_HANDLED;
 
 	} else if(dbus_message_is_signal(message, "org.bluez.Adapter", "RemoteDeviceDisconnected")){
@@ -828,6 +844,8 @@ acting_dead_state = 0;
 		if(hw_bluetooth.nr_devices){
 			hw_bluetooth.nr_devices--;
 			if(is_value_visible(bluetooth_widget_page)){ hw_changes |= HW_BLUETOOTH; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+
+			dbus_send_bluetooth_status_signal();
 		}
 
 		return DBUS_HANDLER_RESULT_HANDLED;
@@ -839,6 +857,9 @@ acting_dead_state = 0;
 		if(!hw_bluetooth.headset){
 			hw_bluetooth.headset = 1;
 			if(is_value_visible(volume_widget_page)){ hw_changes |= HW_VOLUME_OUTPUT; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+
+// TODO: send signal to statusbar applet so it can change its icon?
+
 		}
 
 		return DBUS_HANDLER_RESULT_HANDLED;
@@ -850,6 +871,9 @@ acting_dead_state = 0;
 		if(hw_bluetooth.headset){
 			hw_bluetooth.headset = 0;
 			if(is_value_visible(volume_widget_page)){ hw_changes |= HW_VOLUME_OUTPUT; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+
+// TODO: send signal to statusbar applet so it can change its icon?
+
 		}
 
 		return DBUS_HANDLER_RESULT_HANDLED;
@@ -864,6 +888,8 @@ acting_dead_state = 0;
 			hw_wifi.enabled = 1;
 			hw_new_event("wifi", &hw_wifi.history_head, &hw_wifi.history_tail, 1);
 			if(is_value_visible(wifi_widget_page)){ hw_changes |= HW_WIFI; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+
+			dbus_send_wifi_status_signal();
 		}
 
 		return DBUS_HANDLER_RESULT_HANDLED;
@@ -876,6 +902,14 @@ acting_dead_state = 0;
 			hw_wifi.enabled = 0;
 			hw_new_event("wifi", &hw_wifi.history_head, &hw_wifi.history_tail, 0);
 			if(is_value_visible(wifi_widget_page)){ hw_changes |= HW_WIFI; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+
+			// enter flight mode and return to normal mode unless explicitly entering flight mode
+			if(hw_wifi.disconnect_unfix == 0){
+				if(cfg_wifi_disconnect_fix){ usleep(250000); hw_wifi.disconnect_unfix = 1; mce_flight_mode(0); }
+			} else
+				hw_wifi.disconnect_unfix = 0;
+
+			dbus_send_wifi_status_signal();
 		}
 
 		return DBUS_HANDLER_RESULT_HANDLED;
@@ -917,6 +951,8 @@ acting_dead_state = 0;
 					if(is_value_visible(wifi_widget_page)){ hw_changes |= HW_WIFI; dbus_send_wakeup(_WAKEUP_REFRESH_); }
 
 					if(!hw_wifi.enabled && hw_entering_flight_mode) mce_flight_mode();
+
+					dbus_send_wifi_status_signal();
 				}
 			}
 		}
@@ -982,6 +1018,8 @@ acting_dead_state = 0;
 					hw_wifi.state = state;
 					hw_wifi.enabled = (state == WIFI_DISCONNECTED ? 0 : 1);
 					if(is_value_visible(wifi_widget_page)){ hw_changes |= HW_WIFI; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+
+					dbus_send_wifi_status_signal();
 				}
 
 				// now get rid of this signal
@@ -1400,3 +1438,26 @@ void dbus_send_battery_capacity_signal( ){
 	dbus_send_signal("/com/qzx/asui", "com.qzx.asui", "battery_capacity", _UINT32_, &capacity, _END_);
 	#endif
 }
+
+void dbus_send_wifi_status_signal( ){
+	#ifdef ARCH_armel
+	unsigned status = (hw_flight_mode ? 0 : (hw_wifi.enabled ? hw_wifi.state+1 : 1));
+	// WIFI_FLIGHT_MODE, WIFI_DISCONNECTED, WIFI_CONNECTING, WIFI_CONNECTED, WIFI_DISCONNECTING
+	dbus_send_signal("/com/qzx/asui", "com.qzx.asui", "wifi_status", _UINT32_, &status, _END_);
+	#endif
+}
+
+void dbus_send_bluetooth_status_signal( ){
+	#ifdef ARCH_armel
+	// BLUETOOTH_OFF, BLUETOOTH_ON, BLUETOOTH_CONNECTED, BLUETOOTH_ON_VISIBLE, BLUETOOTH_CONNECTED_VISIBLE
+	unsigned status = (hw_flight_mode || !hw_bluetooth.enabled ? 0 : 1 + (hw_bluetooth.nr_devices ? 1 : 0) + (hw_bluetooth.visible ? 2 : 0));
+	dbus_send_signal("/com/qzx/asui", "com.qzx.asui", "bluetooth_status", _UINT32_, &status, _END_);
+	#endif
+}
+
+void dbus_send_display_status_signal( ){
+	#ifdef ARCH_armel
+	unsigned status = hw_screen.visible;
+	dbus_send_signal("/com/qzx/asui", "com.qzx.asui", "display_status", _BOOL_, &status, _END_);
+	#endif
+}
