@@ -7,7 +7,7 @@ extern unsigned tick_i;
 //////////////////////////////////////////////////////////////////////////
 
 void draw_masked_image( s_image *image, int x, int y );
-void draw_image( s_image *image, int x, int y );
+void draw_themed_image( s_image *image, int x, int y );
 
 void not_ready( int x, int y );
 
@@ -20,12 +20,11 @@ void draw_scrollbar( s_image *image, int x, int y, unsigned position, unsigned m
 unsigned click_scrollbar( int x, unsigned min, unsigned max );
 void draw_step_buttons( int x, int y, unsigned long_decrement, unsigned long_increment );
 
-void highlight_long_press( int x, int y, int width, int height );
+void highlight_long_press( int x, int y, int width, int height, unsigned secondary );
 void unhighlight_long_press( );
-void reset_long_press( );
 
-void highlight_selected_area( int x, int y, int width, int height );
-void unhighlight_selected_area( int x, int y, int width, int height );
+void highlight_selected_area( int x, int y, int width, int height, unsigned secondary );
+void unhighlight_selected_area( );
 
 void init_ticks( int x, int y, XColor *color );
 void draw_tick( );
