@@ -212,9 +212,11 @@ void dsme_read( ){
 
 				debug_write("[DSME] screen off");
 			}
-			if(hw_screen.visible != visible)
+			if(hw_screen.visible != visible){
 				hw_new_event("screen", &hw_screen.history_head, &hw_screen.history_tail, visible);
-			hw_screen.visible = visible;
+				hw_screen.visible = visible;
+				dbus_send_display_status_signal();
+			}
 			// don't need to wakeup main loop for this, dbus unlock signal or X screen/key events will wake it
 
 			if(hw_screen.dimmed != dimmed){
