@@ -204,6 +204,7 @@ static void hw_query_volume( ){
 	// called from hw_refresh()
 
 	#ifdef ARCH_armel
+	if(conf_get_bool(conf_ignore_master_volume)) return; // statusbar applet is playing a sound at different volume level, ignore
 	int volume = conf_get_int(conf_volume);
 	if(hw_volume != volume){ hw_volume = volume; hw_changes |= HW_VOLUME; }
 	#else
@@ -281,13 +282,6 @@ static const char *touchscreen_control = "/sys/devices/platform/omap2_mcspi.1/sp
 #define KEYPAD_CONTROL_N810 "/sys/devices/platform/i2c_omap.2/i2c-0/0-0045/disable_kp"
 static const char *keypad_control;
 
-#ifdef DEBUG
-void debug_tklock_state( const char *section ){
-	debug_write("[%s] tklock=%u, tc=%u, kc=%u, puk=%u", section, hw_screen.locked,
-		get_file_value(touchscreen_control), get_file_value(keypad_control), cfg_power_unlock_key);
-}
-#endif
-
 static void sl_init( ){
 	if(!hw_screen.visible && !hw_screen.locked && !conf_get_screen_autolock_value()){
 		if(cfg_start_visible)
@@ -350,12 +344,24 @@ static void mce_get_flight_callback( DBusPendingCall *pending, void *user_data )
 		unsigned flight = (strcmp(mode, _mce_device_NORMAL_) ? 1 : 0);
 		if(hw_flight_mode != flight){ hw_flight_mode = flight; hw_changes |= HW_FLIGHT_MODE; }
 		hw_initialized &= ~HWIS_MCE_FLIGHT;
+
+		dbus_send_wifi_status_signal();
+		dbus_send_bluetooth_status_signal();
 	}
 	dbus_message_unref(message);
 }
 #endif
 
-void mce_init( ){
+#ifdef DEBUG
+void debug_tklock_state( const char *section ){
+	#ifdef ARCH_armel
+	debug_write("[%s] tklock=%u, tc=%u, kc=%u, puk=%u", section, hw_screen.locked,
+		get_file_value(touchscreen_control), get_file_value(keypad_control), cfg_power_unlock_key);
+	#endif
+}
+#endif
+
+static void mce_init( ){
 	// this combo won't produce a "screen active" notification but will produce a "screen locked"
 	// if the app is restarted while the screen is locked
 	hw_screen.visible = 1;
@@ -564,8 +570,9 @@ void mce_normal_mode( ){
 	#endif
 }
 
-void mce_flight_mode( ){
+void mce_flight_mode( unsigned explicit ){
 	#ifdef ARCH_armel
+	if(explicit && cfg_wifi_disconnect_fix) hw_wifi.disconnect_unfix = -1;
 	if(hw_wifi.enabled){
 		// disconnect wifi so ICD won't run in flight mode
 		hw_wifi_disconnect();
@@ -647,9 +654,9 @@ static void bme_update_drain( ){
 	double timestamp = hw_battery.history_tail->timestamp;
 	float capacity = hw_battery.history_tail->capacity;
 	float current_capacity = capacity;
+	s_battery_history *history;
 
 	// last hour
-	s_battery_history *history;
 	for(history = hw_battery.history_tail; history != NULL; history = history->prev){
 		timestamp = history->timestamp;
 		capacity = history->capacity;
@@ -1171,6 +1178,8 @@ static void bluez_get_mode_callback( DBusPendingCall *pending, void *user_data )
 		}
 		hw_new_event("bluetooth", &hw_bluetooth.history_head, &hw_bluetooth.history_tail, hw_bluetooth.enabled);
 		hw_initialized &= ~HWIS_BLUETOOTH_MODE;
+
+		dbus_send_bluetooth_status_signal();
 	}
 	dbus_message_unref(message);
 }
@@ -1221,6 +1230,8 @@ static void bluez_get_devices_callback( DBusPendingCall *pending, void *user_dat
 		hw_initialized &= ~HWIS_BLUETOOTH_DEVICES;
 
 		dbus_free_string_array(devices);
+
+		dbus_send_bluetooth_status_signal();
 	}
 	dbus_message_unref(message);
 }
@@ -1238,6 +1249,9 @@ static void bluez_get_headset_state_callback( DBusPendingCall *pending, void *us
 			if(is_value_visible(volume_widget_page)){ hw_changes |= HW_VOLUME_OUTPUT; dbus_send_wakeup(_WAKEUP_REFRESH_); }
 		}
 		hw_initialized &= ~HWIS_BLUETOOTH_HEADSET;
+
+// TODO: send signal to statusbar applet so it can change its icon?
+
 	}
 	dbus_message_unref(message);
 }
@@ -1362,6 +1376,7 @@ static const char *_icd_path_ = "/com/nokia/icd";
 static const char *_icd_interface_ = "com.nokia.icd";
 
 static const char *_icd_get_ipinfo_ = "get_ipinfo"; // method
+//static const char *_icd_get_statistics_ = "get_statistics"; // method		
 
 static const char *_icdui_service_ = "com.nokia.icd_ui";
 static const char *_icdui_path_ = "/com/nokia/icd_ui";
@@ -1373,6 +1388,60 @@ static const char *_icd2_interface_ = "com.nokia.icd2";
 
 static const char *_icd2_state_req_ = "state_req"; // method
 
+/*
+static const char *_wlancond_service_ = "com.nokia.wlancond";				
+static const char *_wlancond_path_ = "/com/nokia/wlancond/request";			
+static const char *_wlancond_interface_ = "com.nokia.wlancond.request";		
+
+static const char *_wlancond_status_ = "status"; // method					
+*/
+
+/*
+static void icd_get_statistics_callback( DBusPendingCall *pending, void *user_data ){
+	DBusMessage *message = dbus_pending_call_steal_reply(pending);
+	dbus_pending_call_unref(pending);
+	char *name;
+	unsigned unknown1, timestamp, unknown3, unknown4, rx_bytes, tx_bytes;
+	debug_write("[DBUS] callback: icd_get_statistics_callback");
+	if(dbus_message_get_args(message, &dbus_error, _STRING_, &name,
+		_UINT32_, &unknown1, _UINT32_, &timestamp, _UINT32_, &unknown3, _UINT32_, &unknown4, _UINT32_, &rx_bytes, _UINT32_, &tx_bytes, _END_))
+	{
+printf("ICD statistics: %u -- %u -- %u %u -- %u %u\n", timestamp, unknown1, unknown3, unknown4, rx_bytes, tx_bytes);
+//	name		= 
+//	unknown1	= increases with time (packet count?)
+//	timestamp	= appears to be a timestamp and always matches the value from WLANCOND
+//	unknown3	= always zero
+//	unknown4	= always zero
+//	rx_bytes	= incoming bandwidth for wifi session
+//	tx_bytes	= outgoing bandwidth for wifi session
+	}
+	dbus_message_unref(message);
+}
+*/
+/*
+static void wlancond_status_callback( DBusPendingCall *pending, void *user_data ){
+	DBusMessage *message = dbus_pending_call_steal_reply(pending);
+	dbus_pending_call_unref(pending);
+	char *ssid, *name2, *name3;
+	unsigned timestamp, channel, unknown3, unknown4;
+	int ssid_len, name2_len;
+	debug_write("[DBUS] callback: wlancond_status_callback");
+	if(dbus_message_get_args(message, &dbus_error, _ARRAY_, DBUS_TYPE_BYTE, &ssid, &ssid_len, _ARRAY_, DBUS_TYPE_BYTE, &name2, &name2_len,
+		_UINT32_, &timestamp, _UINT32_, &channel, _UINT32_, &unknown3, _UINT32_, &unknown4, _STRING_, &name3, _END_))
+	{
+printf("WLAN status:    %u -- %u -- %u %u\n", timestamp, channel, unknown3, unknown4);
+//	ssid		= ssid
+//	name2		= ""
+//	timestamp	= appears to be a timestamp and always matches the value from ICD
+//	channel		= wifi channel
+//	unknown3	= usually 1000001, was 2000001 once
+//	unknown4	= always 16
+//	name3		= "wlan0"
+	}
+	dbus_message_unref(message);
+}
+*/
+
 static void icd2_state_req_callback( DBusPendingCall *pending, void *user_data ){
 	DBusMessage *message = dbus_pending_call_steal_reply(pending);
 	dbus_pending_call_unref(pending);
@@ -1409,6 +1478,7 @@ static void wifi_init( ){
 	hw_wifi.name = NULL;
 	hw_wifi.address = NULL;
 	hw_wifi.state = WIFI_DISCONNECTED;
+	hw_wifi.disconnect_unfix = 0;
 
 	hw_wifi.history_head = hw_wifi.history_tail = NULL;
 
@@ -1619,6 +1689,15 @@ void hw_refresh( ){
 			#endif
 		}
 
+/*
+		if(wifi_widget_page == primary_widgets){
+			#ifdef ARCH_armel
+//			dbus_recv_method(icd_get_statistics_callback, _icd_service_, _icd_path_, _icd_interface_, _icd_get_statistics_, _END_);
+			dbus_recv_method(wlancond_status_callback, _wlancond_service_, _wlancond_path_, _wlancond_interface_, _wlancond_status_, _END_);
+			#endif
+		}
+*/
+
 		if(primary_widgets){
 			// an external rotation resulting in a screen resize will produce an X event that is used to recreate window with update_orientation()
 			// however, no event is generated if the screen merely flips orientation so it must be checked and corrected here so the rotation icons can be redrawn
