@@ -76,6 +76,9 @@
 // shutdown dialog
 #include "button_reboot_shutdown.h"
 
+// screenshot delay timer
+#include "button_screenshot.h"
+
 ////////////////////////////////////////////////////////////////////////// GLOBALS
 
 // device model
@@ -213,7 +216,7 @@ static void _draw_button( unsigned force, int x, int y, s_button *button ){
 				} else color = fg1_color;
 				draw_text((char *)button->name, 12, color, x+39, y+45, ALIGN_CENTER, ALIGN_BASELINE, 0);
 			} else {
-				x_set_color(line_color);
+				x_set_color(line1_color);
 				x_fill_ellipse(x+34,y+34, 11,11);
 			}
 		}
@@ -226,7 +229,7 @@ static void _move_button( s_button *button, unsigned cell ){
 	if((selected_cell.button == &button_rotate1 || selected_cell.button == &button_rotate2 || selected_cell.button == &button_rotate3) && cell >= 10)
 		return;
 
-	highlight_selected_area(widget_x, widget_y, 80, 80);
+	highlight_selected_area(widget_x, widget_y, 80, 80, 0);
 
 	// swap button with another button or an empty cell
 	button_cells[selected_cell.cell] = button;
@@ -252,9 +255,8 @@ static void _click_button( s_button *button, unsigned cell ){
 
 		// clear selection or move selected button
 		if(button == selected_cell.button){
-			highlight_long_press(widget_x, widget_y, 80,80);
+			highlight_long_press(widget_x, widget_y, 80,80, 0); // highlight removed when window is redrawn
 			selected_cell.button = NULL;
-			reset_long_press(); // highlight removed when window is redrawn
 			draw_window(1);
 		} else
 			_move_button(button, cell);
@@ -303,7 +305,7 @@ static void _draw_widget( unsigned force, int x, int y, s_widget_cell *widget_ce
 				widget_cell->left->draw(force, x, y);
 		}
 		if(force){
-			x_set_color(line_color);
+			x_set_color(line1_color);
 			x_draw_line(x+199, y, x+199, y+99);
 		}
 		if(widget_cell->right){
@@ -327,9 +329,9 @@ static void _move_widget( s_widget_cell *widget_cell, unsigned cell, unsigned su
 	e_widget_size src_size = selected_cell.widget_cell->size, dst_size = widget_cell->size;
 
 	if(src_size == FULL_WIDGET)
-		highlight_selected_area(widget_x, widget_y, 399, 99);
+		highlight_selected_area(widget_x, widget_y, 399, 99, 0);
 	else
-		highlight_selected_area(widget_x+(subcell ? 200 : 0), widget_y, 199, 99);
+		highlight_selected_area(widget_x+(subcell ? 200 : 0), widget_y, 199, 99, 0);
 
 
 	if(src_size == FULL_WIDGET && (dst_size == FULL_WIDGET || dst_size == EMPTY_CELL)){
@@ -400,11 +402,10 @@ static void _click_widget( s_widget_cell *widget_cell, unsigned cell ){
 		unsigned subcell = (click.x >= 200 ? 1 : 0);
 		if(widget_cell == selected_cell.widget_cell && (widget_cell->size == FULL_WIDGET || (widget_cell->size == HALF_WIDGET && selected_cell.subcell == subcell))){
 			if(widget_cell->size == FULL_WIDGET)
-				highlight_long_press(widget_x, widget_y, 399,99);
+				highlight_long_press(widget_x, widget_y, 399,99, 0); // highlight removed when window is redrawn
 			else
-				highlight_long_press(widget_x+(subcell ? 200 : 0), widget_y, 199,99);
+				highlight_long_press(widget_x+(subcell ? 200 : 0), widget_y, 199,99, 0); // highlight removed when window is redrawn
 			selected_cell.widget_cell = NULL;
-			reset_long_press(); // highlight removed when window is redrawn
 			draw_window(1);
 		} else
 			_move_widget(widget_cell, cell, subcell);
@@ -467,7 +468,7 @@ static void _click_widget( s_widget_cell *widget_cell, unsigned cell ){
 static void draw_self_rss( unsigned force, int x, int y, unsigned landscape ){
 	if(force || svc_changes & SVC_SELF){
 		char buffer[20];
-		x_set_color(bg_color);
+		x_set_color(secondary_bg_color);
 		snprintf(buffer, sizeof(buffer), "%u", svc_self_size);
 		if(landscape){
 			x_fill_rectangle(x-48,y, 96,20);
@@ -545,20 +546,20 @@ void error_write( const char *format, ... ){
 /*
 static void draw_button_guide_lines( unsigned rows ){
 	if(rows == 2){
-		x_set_color(line_color);
+		x_set_color(line1_color);
 		x_draw_line(0,28, window.width,28);
 		x_draw_line(0,52, window.width,52);
-		x_set_color(selection_color);
+		x_set_color(blue_color);
 		x_set_style(1, LineOnOffDash);
 		x_draw_line(0,33, window.width,33);
 		x_draw_line(0,57, window.width,57);
 		x_reset_style();
 	} else if(rows == 3){
-		x_set_color(line_color);
+		x_set_color(line1_color);
 		x_draw_line(0,20, window.width,20);
 		x_draw_line(0,40, window.width,40);
 		x_draw_line(0,60, window.width,60);
-		x_set_color(selection_color);
+		x_set_color(blue_color);
 		x_set_style(1, LineOnOffDash);
 		x_draw_line(0,25, window.width,25);
 		x_draw_line(0,45, window.width,45);
@@ -583,7 +584,7 @@ static void _draw_selection_button( int x, int y ){
 			s_widget *widget = (selected_cell.subcell == 0 ? selected_cell.widget_cell->left : selected_cell.widget_cell->right);
 			x_set_color(selection_color);
 			x_fill_rectangle(x,y, 80,80);
-			draw_text((char *)widget->name, 12, bg_color, x+40,y+40, ALIGN_CENTER, ALIGN_MIDDLE, 76);
+			draw_text((char *)widget->name, 12, secondary_bg_color, x+40,y+40, ALIGN_CENTER, ALIGN_MIDDLE, 76);
 			draw_text((widget->size == FULL_WIDGET ? "FULL" : "HALF"), 8, fg1_color, x+75,y+5, ALIGN_RIGHT, ALIGN_TOP, 0);
 		}
 	} else {
@@ -591,7 +592,7 @@ static void _draw_selection_button( int x, int y ){
 		if(selected_cell.button && ((primary_widgets && selected_cell.cell > 9) || (!primary_widgets && selected_cell.cell < 10))){
 			x_set_color(selection_color);
 			x_fill_rectangle(x,y, 80,80);
-			draw_text((char *)selected_cell.button->name, 12, bg_color, x+40,y+40, ALIGN_CENTER, ALIGN_MIDDLE, 76);
+			draw_text((char *)selected_cell.button->name, 12, secondary_bg_color, x+40,y+40, ALIGN_CENTER, ALIGN_MIDDLE, 76);
 			// hw_refresh() needs the rotation buttons to stay on primary page so it can call check_orientation()
 			if((selected_cell.button == &button_rotate1 || selected_cell.button == &button_rotate2 || selected_cell.button == &button_rotate3) && !primary_widgets)
 				draw_text("CAN'T MOVE", 8, fg1_color, x+75,y+5, ALIGN_RIGHT, ALIGN_TOP, 0);
@@ -654,12 +655,6 @@ void draw_window( unsigned force ){
 
 	} else {
 
-		if(force){
-			// window background
-			x_set_color(bg_color);
-			x_fill_rectangle(0, 0, window.width, window.height);
-		}
-
 		// widgets and buttons
 		if(window.width > window.height){
 			// LANDSCAPE
@@ -674,7 +669,13 @@ void draw_window( unsigned force ){
 			// half widgets are 199x99 with a clickable line on the right side of each (invisible for widgets in right subcell in right column)
 
 			if(force){
-				x_set_color(line_color);
+				// window background
+				x_set_color(bg_color);
+				x_fill_rectangle(0, 81, window.width, window.height-81);
+				x_set_color(secondary_bg_color);
+				x_fill_rectangle(0, 0, window.width, 80);
+
+				x_set_color(line1_color);
 				if(button_edit_mode){
 					// separator between buttons and widgets
 					x_draw_line(0, 80, 799, 80);
@@ -702,9 +703,9 @@ void draw_window( unsigned force ){
 
 /*
 // button guide lines
-x_set_color(line_color);
+x_set_color(line1_color);
 x_draw_line(0,40, window.width,40);
-x_set_color(selection_color);
+x_set_color(blue_color);
 x_set_style(1, LineOnOffDash);
 x_draw_line(0,27, window.width,27);
 x_draw_line(0,59, window.width,59);
@@ -773,7 +774,13 @@ x_reset_style();
 			// half widgets are 199x99 with a clickable line on the right side of each
 
 			if(force){
-				x_set_color(line_color);
+				// window background
+				x_set_color(bg_color);
+				x_fill_rectangle(0, 0, window.width-81, window.height);
+				x_set_color(secondary_bg_color);
+				x_fill_rectangle(400, 0, 80, window.height);
+
+				x_set_color(line1_color);
 				// separator between buttons and widgets
 				x_draw_line(399, 0, 399, 799);
 				if(button_edit_mode){
@@ -852,15 +859,14 @@ static void click_button( unsigned button ){
 			if(widget_edit_mode){
 				if(click.type != CLICK_SHORT || selected_cell.widget_cell == NULL) return;
 
-				highlight_long_press(widget_x, widget_y, 80,80);
+				highlight_long_press(widget_x, widget_y, 80,80, 0); // highlight removed when window is redrawn
 				selected_cell.widget_cell = NULL;
 			} else {
 				if(click.type != CLICK_SHORT || selected_cell.button == NULL) return;
 
-				highlight_long_press(widget_x, widget_y, 80,80);
+				highlight_long_press(widget_x, widget_y, 80,80, 0); // highlight removed when window is redrawn
 				selected_cell.button = NULL;
 			}
-			reset_long_press();
 
 			// redraw window
 			draw_window(1);
@@ -870,7 +876,7 @@ static void click_button( unsigned button ){
 
 			if(click.type != CLICK_SHORT) return;
 
-			highlight_selected_area(widget_x, widget_y, 80,80);
+			highlight_selected_area(widget_x, widget_y, 80,80, 1);
 			if(widget_edit_mode){
 				widget_edit_mode = 0;
 				selected_cell.widget_cell = NULL;
@@ -951,6 +957,8 @@ static void highlight_button_bar( ){
 	// this usleep also fixes a problem where X wouldn't update the window contents for long periods of time
 }
 
+#define WINDOW_SLIDE_DELTA 15
+
 static void screen_event( ){
 	if(current_dialog_draw){
 
@@ -964,6 +972,8 @@ static void screen_event( ){
 			if(click.start_y < 80){
 				// buttons
 				if(click.type == CLICK_DRAG && click.start_y + click.offset_y < 80 && (click.offset_x >= 200 || click.offset_x <= -200)){
+					if(window.y) window_slide(0,0);
+
 					// change page with a horizontal swipe along button bar (25% length)
 					highlight_button_bar();
 					primary_widgets = (primary_widgets ? 0 : 1);
@@ -973,10 +983,19 @@ static void screen_event( ){
 					// throttle taps when switching pages
 					throttle_taps(&ui_throttle);
 					finalize_tap(ui_throttle);
-				} else if(click.type == CLICK_DRAG && click.offset_x >= -100 && click.offset_x <= 100 && click.start_y + click.offset_y > 90){
-					// hide the window with a vertical swipe from button bar to widgets (10 pixels into widgets, +-100 pixel horizontal deviation)
-					window_unmap();
+				} else if(click.offset_x >= -100 && click.offset_x <= 100 && click.start_y + click.offset_y > 80){
+					if(click.type == CLICK_DRAG && click.start_y + click.offset_y > 90){
+						// hide the window with a vertical swipe from button bar to widgets (10 pixels into widgets, +-100 pixel horizontal deviation)
+						window_unmap(1);
+					} else if(click.type == CLICK_MOTION){
+						// slide the window down as finger moves
+						int y = (click.start_y + click.offset_y) - 80;
+						if(y > window.y+WINDOW_SLIDE_DELTA || y < window.y-WINDOW_SLIDE_DELTA)
+							window_slide(0, y);
+					}
 				} else {
+					if(window.y) window_slide(0,0);
+
 					uint button = click.start_x / 80;
 					widget_x = (click.start_x / 80)*80;
 					widget_y = 0;
@@ -1015,7 +1034,9 @@ static void screen_event( ){
 			// PORTRAIT
 			if(click.start_x >= 400){
 				// buttons
-				if(click.type == CLICK_DRAG && click.start_x + click.offset_x >= 400 && click.start_y + click.offset_y >= 200){
+				if(click.type == CLICK_DRAG && click.start_x + click.offset_x >= 400 && (click.offset_y >= 200 || click.offset_y <= -200)){
+					if(window.x) window_slide(0,0);
+
 					// change page with a vertical swipe along button bar (25% length)
 					highlight_button_bar();
 					primary_widgets = (primary_widgets ? 0 : 1);
@@ -1025,10 +1046,19 @@ static void screen_event( ){
 					// throttle taps when switching pages
 					throttle_taps(&ui_throttle);
 					finalize_tap(ui_throttle);
-				} else if(click.type == CLICK_DRAG && click.offset_y >= -100 && click.offset_y <= 100 && click.start_x + click.offset_x < 390){
-					// hide the window with a horizontal swipe from button bar to widgets (10 pixels into widgets, +-100 pixel vertical deviation)
-					window_unmap();
+				} else if(click.offset_y >= -100 && click.offset_y <= 100 && click.start_x + click.offset_x < 400){
+					if(click.type == CLICK_DRAG && click.start_x + click.offset_x < 390){
+						// hide the window with a horizontal swipe from button bar to widgets (10 pixels into widgets, +-100 pixel vertical deviation)
+						window_unmap(1);
+					} else if(click.type == CLICK_MOTION){
+						// slide the window left as finger moves
+						int x = (click.start_x + click.offset_x) - 400;
+						if(x < window.x-WINDOW_SLIDE_DELTA || x > window.x+WINDOW_SLIDE_DELTA)
+							window_slide(x,0);
+					}
 				} else {
+					if(window.x) window_slide(0,0);
+
 					uint button = click.start_y / 80;
 					widget_x = 400;
 					widget_y = (click.start_y / 80)*80;
@@ -1151,17 +1181,17 @@ void press_lock_button( unsigned action ){
 		break;
 	case 1: // lock
 		debug_write("    lock button action: lock");
-		if(cfg_lock_and_close && !hw_device_locked) window_unmap();
+		if(cfg_lock_and_close && !hw_device_locked) window_unmap(1);
 		mce_lock_screen(1);
 		break;
 	case 2: // lock&blank
 		debug_write("    lock button action: lock&blank");
-		if(cfg_lock_and_close && !hw_device_locked) window_unmap();
+		if(cfg_lock_and_close && !hw_device_locked) window_unmap(0);
 		mce_lock_and_blank_screen();
 		break;
 	case 3: // blank
 		debug_write("    lock button action: blank");
-		if(cfg_lock_and_close && !hw_device_locked) window_unmap();
+		if(cfg_lock_and_close && !hw_device_locked) window_unmap(0);
 		mce_blank_screen();
 		break;
 	case 4: // secure
@@ -1234,14 +1264,14 @@ static void main_keypress_handler( ){
 		if(keypress.type == KEY_SHORT){
 			if(widget_edit_mode || button_edit_mode){
 				// exit edit mode
-				highlight_selected_area(window.width-80, (window.width > window.height ? 0 : window.height-80), 80,80); // highlight removed when window is redrawn
+				highlight_selected_area(window.width-80, (window.width > window.height ? 0 : window.height-80), 80,80, 1); // highlight removed when window is redrawn
 				widget_edit_mode = 0;
 				selected_cell.widget_cell = NULL;
 				button_edit_mode = 0;
 				selected_cell.button = NULL;
 				draw_window(1); // refresh, force to clear highlight
 			} else
-				window_unmap();
+				window_unmap(1);
 		}
 		break;
 	case KEYCODE_MENU:
@@ -1347,8 +1377,8 @@ void detect_runlevel_transition( ){
 			#ifdef DEBUG
 			debug_runlevel = runlevel;
 			#endif
-			window_unmap(); // always unmap, the remap will grab keyboard and pointer
-			if(hw_device_locked || cfg_start_visible) window_map();
+			window_unmap(1); // always unmap, the remap will grab keyboard and pointer
+			if(hw_device_locked || cfg_start_visible) window_map(1);
 //			alarm_resume(1);
 		}
 	}
@@ -1455,7 +1485,9 @@ if(runlevel == 2){
 	}
 
 	// initialize gconf settings
+	debug_write("INIT: config");
 	config_init();
+	debug_write("INIT: cp screen settings");
 	hw_init_cp_screen_settings();
 
 	// use minimal UI in runlevel 5
@@ -1471,6 +1503,7 @@ if(runlevel == 2){
 	// initialize dsme socket
 	int dsme_fd;
 	#ifdef ARCH_armel
+	debug_write("INIT: dsme");
 	if(!(dsme_fd = dsme_init())){
 		clean_up();
 		return EXIT_FAILURE;
@@ -1483,6 +1516,7 @@ if(runlevel == 2){
 	// initialize power button socket
 	int powerbutton_fd;
 	#ifdef ARCH_armel
+	debug_write("INIT: power button");
 	if(!(powerbutton_fd = powerbutton_init())){
 		clean_up();
 		return EXIT_FAILURE;
@@ -1496,6 +1530,7 @@ if(runlevel == 2){
 	alarm_init();
 
 	// initialize dbus thread
+	debug_write("INIT: dbus");
 	if(!(dbus_fd = dbus_init())){
 		clean_up();
 		return EXIT_FAILURE;
@@ -1503,6 +1538,7 @@ if(runlevel == 2){
 	clean_up_stage++;
 
 	// initialize the window
+	debug_write("INIT: window");
 	if(!window_init()){
 		clean_up();
 		return EXIT_FAILURE;
@@ -1510,7 +1546,9 @@ if(runlevel == 2){
 	clean_up_stage++;
 
 	// initialize services and hardware
+	debug_write("INIT: services");
 	svc_init();
+	debug_write("INIT: hardware");
 	hw_init();
 
 	////////////////////
@@ -1518,20 +1556,24 @@ if(runlevel == 2){
 	primary_widgets = 1;
 	widget_edit_mode = 0;
 	button_edit_mode = 0;
-	if(cfg_start_visible) window_map();
+	if(cfg_start_visible){ debug_write("INIT: map window"); window_map(1); }
 
 	#ifdef ARCH_armel
+	debug_write("INIT: I am SystemUI!!!");
 	dbus_send_signal("/com/nokia/system_ui/signal", "com.nokia.system_ui.signal", "system_ui_started", _END_);
 	#endif
 
+	debug_write("INIT: resume alarm?");
 	alarm_resume(0); // resume an alarm if ASUI was killed
 
 	////////////////////
 
+	debug_write("INIT: X11");
 	int x_fd = ConnectionNumber(window.display);
 	unsigned main_refresh_all = 0, infinite_timeout = (!window.is_mapped || !hw_screen.visible ? 1 : 0);
 	double next_timer_duration = MAIN_REFRESH_PERIOD, last_focus_grab = 0.0;
 
+	debug_write("INIT: timers");
 	s_timer refresh_timer, button_timer, key_timer, brightness_timer;
 	set_timer(&refresh_timer, MAIN_REFRESH_PERIOD);
 	set_timer(&button_timer, BUTTON_CHECK_PERIOD);				button_callback = NULL; /* global */		click.button_down = 0; /* global */
@@ -1539,9 +1581,12 @@ if(runlevel == 2){
 	set_timer(&brightness_timer, BRIGHTNESS_CHECK_PERIOD);
 	/* global motion_timer, don't need to clear it */			motion_callback = NULL; /* global */		unsigned has_motion = 0;
 
+	screenshot_waiting = 0;
+
 	double_tap_data.zone = Z_NULL; /* static global */
 
 	// event loop
+	debug_write("INIT: start main loop...");
 	for(;;){
 		unsigned runaway_event_counter;
 		struct timeval timeout;
@@ -1682,7 +1727,6 @@ if(runlevel == 2){
 						click.type = CLICK_SHORT;
 					}
 
-					reset_long_press();
 					screen_event();
 
 				} else {
@@ -1832,14 +1876,14 @@ if(runlevel == 2){
 
 					// show the window
 					debug_write("    message SHOW");
-					if(!window.is_mapped) window_map();
+					if(!window.is_mapped) window_map(1);
 					break;
 
 				case _WAKEUP_HIDE_:
 
 					// hide the window
 					debug_write("    message HIDE");
-					if(window.is_mapped && !hw_device_locked && current_dialog_draw != draw_alarm) window_unmap();
+					if(window.is_mapped && !hw_device_locked && current_dialog_draw != draw_alarm) window_unmap(1);
 					break;
 
 				case _WAKEUP_ROTATE_LEFT_:
@@ -1896,14 +1940,14 @@ if(runlevel == 2){
 						} else {
 							devlock_map_state = window.is_mapped; // save window state
 							if(!window.is_mapped)
-								window_map();
+								window_map(1);
 							if(current_dialog_draw != draw_fullsreen_clock && current_dialog_draw != draw_flashlight && current_dialog_draw != draw_alarm)
 								secureShowKeypad();
 						}
 					} else {
 						debug_write("    message DEVICE UNLOCK");
+						if(!devlock_map_state) window_unmap(1); // restore window state
 						secureHideKeypad();
-						if(!devlock_map_state) window_unmap(); // restore window state
 					}
 					break;
 
@@ -1911,6 +1955,12 @@ if(runlevel == 2){
 
 					// ignore it -- used to wake main loop and refresh window
 					debug_write("    message WAKEUP");
+
+					if(hw_changes & HW_THEME_FILE){
+						window_reload(0);
+						hw_changes &= ~HW_THEME_FILE | HW_THEME; // replace THEME_FILE with THEME
+					}
+
 					break;
 
 				case _WAKEUP_SHUTDOWN_:
@@ -1942,7 +1992,7 @@ if(runlevel == 2){
 							if(current_dialog_draw != NULL)
 								dialog_close(); // close the current dialog, most likely the shutdown dialog
 							else if(!window.is_mapped)
-								window_map(); // map ASUI if not already mapped
+								window_map(0); // map ASUI if not already mapped
 							else
 								draw_window(1);
 						}
@@ -1985,7 +2035,7 @@ if(runlevel == 2){
 			if(current_time - hw_screen.visible_time > 1.0)
 				window_close_blank(); // screen has been on for more than one second, close the blank window
 
-		if(hw_device_locked && !window.is_mapped) window_map();
+		if(hw_device_locked && !window.is_mapped) window_map(1);
 
 		// query ambient light sensor and update theme when a dialog is open
 		// dialogs that don't use refresh timers depend on dbus signals to wake the loop and won't respond to light changes until this happens
@@ -2111,7 +2161,6 @@ if(runlevel == 2){
 							// screen is being pressed but has exceeded the long press duration
 							// location has also drifted and is can no longer be a long press
 							unhighlight_long_press();
-							reset_long_press();
 						} else {
 							// screen is being pressed but has exceeded the long press duration
 							// refresh the highlight in case it was erased
@@ -2168,10 +2217,9 @@ if(runlevel == 2){
 					// periodically dispatch a callback when as finger moves across screen
 					if(motion_callback()) reset_timer(&motion_timer);
 					has_motion = 0;
-				}
-
-				if(has_motion && (next_timer_duration <= 0.0 || motion_timer.remaining < next_timer_duration))
-					next_timer_duration = motion_timer.remaining;
+				} else
+					if(next_timer_duration <= 0.0 || motion_timer.remaining < next_timer_duration)
+						next_timer_duration = motion_timer.remaining;
 			}
 
 		}
@@ -2196,6 +2244,16 @@ if(runlevel == 2){
 		}
 		#endif
 
+		if(screenshot_waiting){
+			if(is_timer_ready(&screenshot_timer)){
+				take_screenshot();
+			} else {
+				if(next_timer_duration <= 0.0 || screenshot_timer.remaining < next_timer_duration)
+					next_timer_duration = screenshot_timer.remaining;
+				infinite_timeout = 0;
+			}
+		}
+
 		// refresh on dbus signals and method returns
 			// all dialogs should change on HW_THEME
 			// cpu policy dialog needs HW_CPU
