@@ -50,7 +50,7 @@ static unsigned nr_failures;
 static double last_change_time, last_failure_time;
 
 static void draw_button( unsigned button, int x, int y, int w, int h, unsigned size, char *label, XColor *color, void (*callback)() ){
-	x_set_color(line_color);
+	x_set_color(line1_color);
 	x_draw_rectangle(x+16,y+32, w,h);
 	draw_text(label, size, color, x+16+(w>>1),y+32+(h>>1), ALIGN_CENTER, ALIGN_MIDDLE, 0);
 	buttons[button].x = x+16;
@@ -75,7 +75,7 @@ static void backspace( ){
 }
 
 static void insert_digit( unsigned button ){
-	highlight_selected_area(buttons[button].x, buttons[button].y, buttons[button].w+1, buttons[button].h+1);
+	highlight_selected_area(buttons[button].x, buttons[button].y, buttons[button].w+1, buttons[button].h+1, 0);
 	digits[nr_digits] = buttons[button].label[0];
 	nr_digits++;
 	last_change_time = get_utime();
@@ -103,7 +103,7 @@ static void unlock( ){
 static void draw_keypad( unsigned force, int x, int y ){
 	if(force){
 		// draw separator line
-		x_set_color(line_color);
+		x_set_color(line1_color);
 		x_draw_line(0,99, window.width,99);
 	}
 
@@ -115,7 +115,7 @@ static void draw_keypad( unsigned force, int x, int y ){
 
 		char buffer[100];
 		snprintf(buffer, sizeof(buffer), "Disabled for %.0f+ seconds", TRIPLE_FAILURE_TIMEOUT - (current_time - last_failure_time));
-		draw_text(buffer, 36, red_status_color, window.width>>1, 100+((window.height-100)>>1), ALIGN_CENTER, ALIGN_BOTTOM, 0);
+		draw_text(buffer, 36, red_color, window.width>>1, 100+((window.height-100)>>1), ALIGN_CENTER, ALIGN_BOTTOM, 0);
 	} else {
 		if(nr_failures >= 3){
 			nr_failures = 0; // reset after timeout
@@ -135,16 +135,16 @@ static void draw_keypad( unsigned force, int x, int y ){
 
 		// digit bar
 		unsigned d;
-		x_set_color((device_lock_code == NULL ? red_status_color : green_status_color));
+		x_set_color((device_lock_code == NULL ? red_color : green_color));
 		for(d = 0; d < 10; d++){
-			if(d == nr_digits && device_lock_code) x_set_color(gray_status_color);
+			if(d == nr_digits && device_lock_code) x_set_color(gray_color);
 			if(d >= nr_digits || device_lock_code == NULL)
 				x_fill_ellipse(x+41+d*44-4,y+16-4, 9,9); // small gray or red dots
 			else
 				x_fill_ellipse(x+41+d*44-8,y+16-8, 17,17); // large green dots
 		}
 		// buttons
-		XColor *color = (device_lock_code == NULL || nr_digits == 10 ? line_color : white_color);
+		XColor *color = (device_lock_code == NULL || nr_digits == 10 ? line1_color : bold_color);
 		draw_button( 6, x+  0, y+0*87, 100, 71, 60, "7",		color, NULL);
 		draw_button( 3, x+  0, y+1*87, 100, 71, 60, "4",		color, NULL);
 		draw_button( 0, x+  0, y+2*87, 100, 71, 60, "1",		color, NULL);
@@ -155,20 +155,20 @@ static void draw_keypad( unsigned force, int x, int y ){
 		draw_button( 8, x+232, y+0*87, 100, 71, 60, "9",		color, NULL);
 		draw_button( 5, x+232, y+1*87, 100, 71, 60, "6",		color, NULL);
 		draw_button( 2, x+232, y+2*87, 100, 71, 60, "3",		color, NULL);
-		draw_button(10, x+232, y+3*87, 216, 71, 30, "UNLOCK",	(nr_digits ? green_status_color : line_color), unlock);
-		draw_button(11, x+348, y+0*87, 100, 71, 30, "CLR",		(nr_digits ? secondary_selection_color : line_color), clear_code);
-		draw_button(12, x+348, y+1*87, 100,158, 50, "<",		(nr_digits ? secondary_selection_color : line_color), backspace);
+		draw_button(10, x+232, y+3*87, 216, 71, 30, "UNLOCK",	(nr_digits ? green_color : line1_color), unlock);
+		draw_button(11, x+348, y+0*87, 100, 71, 30, "CLR",		(nr_digits ? secondary_selection_color : line1_color), clear_code);
+		draw_button(12, x+348, y+1*87, 100,158, 50, "<",		(nr_digits ? secondary_selection_color : line1_color), backspace);
 		// the indices of last three buttons are used below, DON'T CHANGE THEM!!!
 	}
 }
 
 static void draw_brightness_toggle( unsigned force ){
 	if(force){
-		x_set_color(bg_color);
+		x_set_color(secondary_bg_color);
 		x_fill_rectangle(200,0, 75, 99);
 	}
 
-	x_set_color(red_status_color);
+	x_set_color(red_color);
 	unsigned size = (unsigned)roundf(21.0 + (30.0 * (float)hw_screen.brightness_level / (float)hw_screen.brightness_max));
 	if((size & 1) == 0) size++; // size is always odd
 	unsigned x = 238 - (size>>1);
@@ -187,12 +187,12 @@ static void click_secure_keypad( ){
 
 		if(click.x > (window.width > window.height ? 640 : 320)){
 			// clock widget
-			highlight_selected_area((window.width > window.height ? 640 : 320), 0, 159, 99);
+			highlight_selected_area((window.width > window.height ? 640 : 320), 0, 159, 99, 1);
 			dialog_close();
 			clockFullscreen();
 		} else if(click.x < 200){
 			// battery widget, manually change theme
-			highlight_selected_area(0, 0, 199, 99);
+			highlight_selected_area(0, 0, 199, 99, 1);
 			// if ALS or black then switch to white
 			// if white then switch to black
 			cfg_theme = (cfg_theme < 2 ? 2 : 1);
@@ -201,7 +201,7 @@ static void click_secure_keypad( ){
 			draw_secure_keypad(0);
 		} else if(click.x < 275){
 			// brightness toggle
-			highlight_selected_area(200, 0, 75, 99);
+			highlight_selected_area(200, 0, 75, 99, 1);
 			if(hw_screen.brightness_level >= (unsigned)(cfg_brightness_enhanced ? hw_screen.brightness_max*0.9 : 5)){ // 90% or 5
 				// at high, change to low
 				hw_set_brightness((cfg_brightness_enhanced ? hw_screen.brightness_max*0.1 : 1), 1); // 10% or 1
@@ -222,7 +222,7 @@ static void click_secure_keypad( ){
 			if(click.x >= buttons[b].x && click.x <= buttons[b].x+buttons[b].w && click.y >= buttons[b].y && click.y <= buttons[b].y+buttons[b].h){
 				if(buttons[b].callback){
 					if(nr_digits){
-						highlight_selected_area(buttons[b].x, buttons[b].y, buttons[b].w+1, buttons[b].h+1);
+						highlight_selected_area(buttons[b].x, buttons[b].y, buttons[b].w+1, buttons[b].h+1, 0);
 						buttons[b].callback();
 						draw_secure_keypad(0);
 					}
@@ -248,7 +248,7 @@ static void keypress_secure_keypad( ){
 	case KEYCODE_ESCAPE:
 		// clear all digits
 		if(nr_digits){
-			highlight_selected_area(buttons[11].x, buttons[11].y, buttons[11].w+1, buttons[11].h+1);
+			highlight_selected_area(buttons[11].x, buttons[11].y, buttons[11].w+1, buttons[11].h+1, 0);
 			clear_code();
 			draw_secure_keypad(0);
 		}
@@ -256,7 +256,7 @@ static void keypress_secure_keypad( ){
 	case 22: // backspace
 		// clear last digit
 		if(nr_digits){
-			highlight_selected_area(buttons[12].x, buttons[12].y, buttons[12].w+1, buttons[12].h+1);
+			highlight_selected_area(buttons[12].x, buttons[12].y, buttons[12].w+1, buttons[12].h+1, 0);
 			backspace();
 			draw_secure_keypad(0);
 		}
@@ -265,14 +265,14 @@ static void keypress_secure_keypad( ){
 	case KEYCODE_DPAD_CENTER:
 		// unlock
 		if(nr_digits){
-			highlight_selected_area(buttons[10].x, buttons[10].y, buttons[10].w+1, buttons[10].h+1);
+			highlight_selected_area(buttons[10].x, buttons[10].y, buttons[10].w+1, buttons[10].h+1, 0);
 			unlock();
 			draw_secure_keypad(0);
 		}
 		break;
 	case KEYCODE_HOME:
 		// open fullscreen clock
-		highlight_selected_area((window.width > window.height ? 640 : 320), 0, 159, 99);
+		highlight_selected_area((window.width > window.height ? 640 : 320), 0, 159, 99, 1);
 		dialog_close();
 		clockFullscreen();
 		break;
@@ -294,7 +294,9 @@ void draw_secure_keypad( unsigned force ){
 	if(force || hw_changes & HW_THEME){
 		// dialog background
 		x_set_color(bg_color);
-		x_fill_rectangle(0, 0, window.width, window.height);
+		x_fill_rectangle(0, 100, window.width, window.height-100);
+		x_set_color(secondary_bg_color);
+		x_fill_rectangle(0, 0, window.width, 99);
 		// randomize position
 		if(force) random_offset = (int)(320.0 * (double)rand()/(double)RAND_MAX);
 		else force = 1;
