@@ -318,7 +318,7 @@ void svc_refresh( ){
 						// process exists, update rss and cpu
 
 						process->rss = rss;
-						int cpu = round(100.0*(float)(ticks - process->_ticks) / (float)(ticks_per_second * interval));
+						int cpu = roundf(100.0*(float)(ticks - process->_ticks) / (float)(ticks_per_second * interval));
 						process->cpu = (cpu < 0 ? 0 : (cpu > 100 ? 100 : cpu));
 						process->_ticks = ticks;
 						process->_alive = 1;
@@ -707,24 +707,24 @@ void svc_init_ssh( e_service_action action, unsigned all_servers ){
 	case SVC_STOP:
 		if(all_servers){
 			// svc_ssh_status only tracks current server so just start both
-			system("/etc/init.d/ssh stop");
-			system("/etc/init.d/dropbear stop");
+			if(file_exists("/etc/init.d/ssh")) system("/etc/init.d/ssh stop");
+			if(file_exists("/etc/init.d/dropbear")) system("/etc/init.d/dropbear stop");
 			changed = 1;
 		} else switch(cfg_ssh_server){
-		case SSH_SERVER_OPENSSH: if(svc_ssh_status){ system("/etc/init.d/ssh stop"); changed = 1; } break;
-		case SSH_SERVER_DROPBEAR: if(svc_ssh_status){ system("/etc/init.d/dropbear stop"); changed = 1; } break;
+		case SSH_SERVER_OPENSSH:  if(svc_ssh_status && file_exists("/etc/init.d/ssh")){ system("/etc/init.d/ssh stop"); changed = 1; } break;
+		case SSH_SERVER_DROPBEAR: if(svc_ssh_status && file_exists("/etc/init.d/dropbear")){ system("/etc/init.d/dropbear stop"); changed = 1; } break;
 		default: break;
 		}
 		break;
 	case SVC_START:
 		if(all_servers){
 			// svc_ssh_status only tracks current server so just start both
-			system("/etc/init.d/ssh start");
-			system("/etc/init.d/dropbear start");
+			if(file_exists("/etc/init.d/ssh")) system("/etc/init.d/ssh start");
+			if(file_exists("/etc/init.d/dropbear")) system("/etc/init.d/dropbear start");
 			changed = 1;
 		} else switch(cfg_ssh_server){
-		case SSH_SERVER_OPENSSH: if(!svc_ssh_status){ system("/etc/init.d/ssh start"); changed = 1; } break;
-		case SSH_SERVER_DROPBEAR: if(!svc_ssh_status){ system("/etc/init.d/dropbear start"); changed = 1; } break;
+		case SSH_SERVER_OPENSSH:  if(!svc_ssh_status && file_exists("/etc/init.d/ssh")){ system("/etc/init.d/ssh start"); changed = 1; } break;
+		case SSH_SERVER_DROPBEAR: if(!svc_ssh_status && file_exists("/etc/init.d/dropbear")){ system("/etc/init.d/dropbear start"); changed = 1; } break;
 		default: break;
 		}
 		break;
