@@ -17,9 +17,11 @@
 #define _END_ DBUS_TYPE_INVALID
 
 #include "main.h"
+#include "statusbar.h"
 #include "battery.h"
 #include "clock.h"
 #include "brightness.h"
+#include "wifi.h"
 #include "rbv_keys.h"
 #include "button_modes.h"
 #include "power_button.h"
@@ -55,6 +57,13 @@ unsigned file_exists( const char *file ){
 
 GConfClient *gconf_client;
 
+// statusbar
+const char *conf_statusbar_battery_enable = "/apps/asui/statusbar_battery_enable";
+const char *conf_statusbar_wifi_enable = "/apps/asui/statusbar_wifi_enable";
+const char *conf_statusbar_bluetooth_enable = "/apps/asui/statusbar_bluetooth_enable";
+const char *conf_statusbar_drain_enable = "/apps/asui/statusbar_drain_enable";
+const char *conf_statusbar_cpu_enable = "/apps/asui/statusbar_cpu_enable";
+
 // battery
 const char *conf_battery_idle_time = "/apps/asui/battery_idle_time";
 const char *conf_memory_format = "/apps/asui/memory_format";
@@ -62,7 +71,6 @@ const char *conf_swap_format = "/apps/asui/swap_format";
 const char *conf_temperature_F = "/apps/asui/temperature_F";
 const char *conf_history_days = "/apps/asui/history_days";
 const char *conf_track_usage_events = "/apps/asui/track_usage_events";
-const char *conf_enable_battery_applet = "/apps/asui/enable_battery_applet";
 // clock
 const char *conf_clock_24h = "/apps/asui/clock_24h";
 const char *conf_clock_ampm = "/apps/asui/clock_ampm";
@@ -74,6 +82,8 @@ const char *conf_popup_noasui = "/apps/asui/popup_noasui";
 // brightness
 const char *conf_brightness_enhanced = "/apps/asui/brightness_enhanced";
 const char *conf_brightness_allow_zero = "/apps/asui/brightness_allow_zero";
+// WiFi
+const char *conf_wifi_disconnect_fix = "/apps/asui/wifi_disconnect_fix";
 // RBV keys
 const char *conf_buttons_rotate = "/apps/asui/buttons_rotate";
 const char *conf_buttons_brightness = "/apps/asui/buttons_brightness";
@@ -110,6 +120,10 @@ const char *conf_short_tap_markers = "/apps/asui/short_tap_markers";
 const char *conf_start_visible = "/apps/asui/start_visible";
 const char *conf_threshold_black = "/apps/asui/threshold_black";
 const char *conf_threshold_white = "/apps/asui/threshold_white";
+const char *conf_theme_file_black = "/apps/asui/theme_file_black";
+const char *conf_theme_file_white = "/apps/asui/theme_file_white";
+const char *conf_window_slide_unmap = "/apps/asui/window_slide_unmap";
+const char *conf_window_slide_map = "/apps/asui/window_slide_map";
 // alarm
 const char *conf_alarm_duration = "/apps/asui/alarm_duration";
 const char *conf_alarm_volume_min = "/apps/asui/alarm_volume_min";
@@ -432,9 +446,12 @@ int main( int argc, char *argv[] ){
 	gtk_widget_show(window_vbox);
 
 	new_scrolling_vbox(window_vbox, &main_box, &main_content_box);
+		new_main_button(main_content_box, "Statusbar", statusbar_click_callback);
+	new_main_separator();
 		new_main_button(main_content_box, "Battery", battery_click_callback);
 		new_main_button(main_content_box, "Clock", clock_click_callback);
 		new_main_button(main_content_box, "Brightness", brightness_click_callback);
+		new_main_button(main_content_box, "WiFi", wifi_click_callback);
 	new_main_separator();
 		new_main_button(main_content_box, "RBV Keys", rbv_keys_click_callback);
 		new_main_button(main_content_box, "Button Modes", button_modes_click_callback);
