@@ -18,9 +18,18 @@ extern GConfClient *gconf_client;
 
 #define conf_get_bool(key) gconf_client_get_bool(gconf_client, key, (GError **)NULL)
 #define conf_get_int(key) gconf_client_get_int(gconf_client, key, (GError **)NULL)
+#define conf_get_string(key) gconf_client_get_string(gconf_client, key, (GError **)NULL)
 
 #define conf_set_bool(key, value) {gconf_client_set_bool(gconf_client, key, value, (GError **)NULL); asui_reload_settings(); }
 #define conf_set_int(key, value) {gconf_client_set_int(gconf_client, key, value, (GError **)NULL); asui_reload_settings(); }
+#define conf_set_string(key, value) {gconf_client_set_string(gconf_client, key, value, (GError **)NULL); asui_reload_settings(); }
+
+// statusbar
+extern const char *conf_statusbar_battery_enable;
+extern const char *conf_statusbar_wifi_enable;
+extern const char *conf_statusbar_bluetooth_enable;
+extern const char *conf_statusbar_drain_enable;
+extern const char *conf_statusbar_cpu_enable;
 
 // battery
 extern const char *conf_battery_idle_time;
@@ -29,7 +38,6 @@ extern const char *conf_swap_format;
 extern const char *conf_temperature_F;
 extern const char *conf_history_days;
 extern const char *conf_track_usage_events;
-extern const char *conf_enable_battery_applet;
 // clock
 extern const char *conf_clock_24h;
 extern const char *conf_clock_ampm;
@@ -41,6 +49,8 @@ extern const char *conf_popup_noasui;
 // brightness
 extern const char *conf_brightness_enhanced;
 extern const char *conf_brightness_allow_zero;
+// WiFi
+extern const char *conf_wifi_disconnect_fix;
 // RBV keys
 extern const char *conf_buttons_rotate;
 extern const char *conf_buttons_brightness;
@@ -77,6 +87,10 @@ extern const char *conf_short_tap_markers;
 extern const char *conf_start_visible;
 extern const char *conf_threshold_black;
 extern const char *conf_threshold_white;
+extern const char *conf_theme_file_black;
+extern const char *conf_theme_file_white;
+extern const char *conf_window_slide_unmap;
+extern const char *conf_window_slide_map;
 // alarm
 extern const char *conf_alarm_duration;
 extern const char *conf_alarm_volume_min;
