@@ -13,6 +13,9 @@
 
 #define APPNAME "Advanced SystemUI"
 
+#define SLIDE_STEP 40
+#define SLIDE_INTERVAL 30000
+
  #include "window.h"
 
 #include <X11/Xproto.h>
@@ -31,10 +34,10 @@
 #include "dbus.h"
 #include "alarm.h"
 
-#include "images/button_toflightmode.xpm"
-#include "images/button_tonormalmode.xpm"
 #include "images/reboot.xpm"
 #include "images/shutdown.xpm"
+#include "images/white_reboot.xpm"
+#include "images/white_shutdown.xpm"
 
 #include "images/icon_brightness.xpm"
 #include "images/icon_volume.xpm"
@@ -50,17 +53,18 @@
 
 s_window window;
 
-#define NR_COLORS 15
-#define BG_COLOR 0
-#define ROTATION_COLOR 6
-XColor *bg_color, *line_color, *innerline_color, *fg1_color, *fg2_color, *white_color, *rotation_color;
-XColor *gray_status_color, *yellow_status_color, *green_status_color, *red_status_color;
-XColor *scrollbar_color, *scroller_color, *selection_color, *secondary_selection_color;
+#define SECONDARY_BG_COLOR 1
+#define ROTATION_COLOR 7
+XColor *bg_color, *secondary_bg_color, *line1_color, *line2_color, *fg1_color, *fg2_color, *bold_color, *rotation_color;
+XColor *gray_color, *yellow_color, *green_color, *red_color, *blue_color;
+XColor *scrollbar_color, *scroller_color, *selection_color, *secondary_selection_color, *button_color, *stepper_color;
+#define NR_COLORS 19
 
-s_image im_toflight, im_tonormal, im_reboot, im_shutdown, im_portrait, im_portrait_inv, im_landscape_inv;
+s_image im_reboot, im_shutdown;
 s_image im_volume, im_brightness;
+s_image im_portrait, im_portrait_inv, im_landscape_inv;
 
-unsigned black_theme;
+unsigned black_theme, theme_is_dark;
 
 int blank_fd, popup_fd;
 unsigned blank_window_open, popup_window_open, drawing_popup_window;
@@ -68,6 +72,8 @@ double popup_window_open_time;
 
 //////////////////////////////////////////////////////////////////////////
 
+unsigned black_is_dark, white_is_dark;
+
 XColor white_colors[NR_COLORS], black_colors[NR_COLORS];
 
 #define MWM_DECORATIONS (1L<<1)
@@ -94,15 +100,20 @@ static void load_masked_xpm( char **image_xpm, s_image *image){
 	image->height = attributes.height;
 }
 
-static void load_xpm( char **image_xpm, s_image *image){
+static void load_themed_xpm( char **black_xpm, char **white_xpm, s_image *image){
 	XpmAttributes attributes;
-	int z, bytes_per_pixel, length;
-	XImage *xi;
+	XpmColorSymbol colors[1] = {{"none", "none", 0}};
+//	int z, bytes_per_pixel, length;
+//	XImage *xi;
 
+	attributes.numsymbols = 1;
+	attributes.colorsymbols = colors;
 	attributes.closeness = 40000;
-	attributes.valuemask = XpmCloseness;
-	XpmCreatePixmapFromData(window.display, window.root_window, image_xpm, &image->black_pixmap, NULL, &attributes);
+	attributes.valuemask = XpmColorSymbols | XpmCloseness;
+	XpmCreatePixmapFromData(window.display, window.root_window, black_xpm, &image->black_pixmap, &image->mask, &attributes);
+	XpmCreatePixmapFromData(window.display, window.root_window, white_xpm, &image->white_pixmap, NULL,         &attributes);
 
+/*
 	// invert image for white theme
 	image->white_pixmap = XCreatePixmap(window.display, window.root_window, attributes.width, attributes.height, window.depth);
 	xi = XGetImage(window.display, image->black_pixmap, 0,0, attributes.width, attributes.height, AllPlanes, ZPixmap);
@@ -116,6 +127,7 @@ static void load_xpm( char **image_xpm, s_image *image){
 	}
 	XPutImage(window.display, image->white_pixmap, window.gc, xi, 0,0, 0,0, attributes.width, attributes.height);
 	XDestroyImage(xi);
+*/
 	image->width = attributes.width;
 	image->height = attributes.height;
 }
@@ -124,105 +136,244 @@ static void load_xpm( char **image_xpm, s_image *image){
 
 static void _rotate_into_image( Pixmap source, float angle, Pixmap destination ){
 	// rotate source into destination
-	{
-		Picture src_picture = None;
-		Picture dst_picture = None;
-		XRenderPictFormat *pictformat = NULL;
-		XRenderPictureAttributes pictattr;
-		XTransform transform;
+	Picture src_picture = None;
+	Picture dst_picture = None;
+	XRenderPictFormat *pictformat = NULL;
+	XRenderPictureAttributes pictattr;
+	XTransform transform;
 
-		pictattr.repeat = RepeatNormal;
-		pictformat = XRenderFindVisualFormat(window.display, window.visual);
-		src_picture = XRenderCreatePicture(window.display, source, pictformat, CPRepeat, &pictattr);
-		dst_picture = XRenderCreatePicture(window.display, destination, pictformat, CPRepeat, &pictattr);
-		transform.matrix[0][0] = (XFixed)(0x10000 * (angle == 0.0 ? -1.0 : 0.0));						//	cosa	 0		-1		 0
-		transform.matrix[0][1] = (XFixed)(0x10000 * (angle == 1.0 ? -1.0 : (angle == 0.0 ? 0.0 : 1.0)));//	sina	-1		 0		 1
-		transform.matrix[0][2] = (XFixed)(0x10000 * 0.0);
-		transform.matrix[1][0] = (XFixed)(0x10000 * angle);												//	-sina	 1		 0		-1
-		transform.matrix[1][1] = (XFixed)(0x10000 * (angle == 0.0 ? -1.0 : 0.0));						//	cosa	 0		-1		 0
-		transform.matrix[1][2] = (XFixed)(0x10000 * 0.0);
-		transform.matrix[2][0] = (XFixed)(0x10000 * 0.0);												// angle =	 1		 0		-1
-		transform.matrix[2][1] = (XFixed)(0x10000 * 0.0);
-		transform.matrix[2][2] = (XFixed)(0x10000 * 1.0);
-		XRenderSetPictureTransform(window.display, src_picture, &transform);
-		XRenderSetPictureFilter(window.display, src_picture, "fast", NULL, 0);
-		XRenderComposite(window.display, PictOpSrc, src_picture, None, dst_picture, 0, 0, 0, 0, 0, 0, 64,64);
+	pictattr.repeat = RepeatNormal;
+	pictformat = XRenderFindVisualFormat(window.display, window.visual);
+	src_picture = XRenderCreatePicture(window.display, source, pictformat, CPRepeat, &pictattr);
+	dst_picture = XRenderCreatePicture(window.display, destination, pictformat, CPRepeat, &pictattr);
+	transform.matrix[0][0] = (XFixed)(0x10000 * (angle == 0.0 ? -1.0 : 0.0));						//	cosa	 0		-1		 0
+	transform.matrix[0][1] = (XFixed)(0x10000 * (angle == 1.0 ? -1.0 : (angle == 0.0 ? 0.0 : 1.0)));//	sina	-1		 0		 1
+	transform.matrix[0][2] = (XFixed)(0x10000 * 0.0);
+	transform.matrix[1][0] = (XFixed)(0x10000 * angle);												//	-sina	 1		 0		-1
+	transform.matrix[1][1] = (XFixed)(0x10000 * (angle == 0.0 ? -1.0 : 0.0));						//	cosa	 0		-1		 0
+	transform.matrix[1][2] = (XFixed)(0x10000 * 0.0);
+	transform.matrix[2][0] = (XFixed)(0x10000 * 0.0);												// angle =	 1		 0		-1
+	transform.matrix[2][1] = (XFixed)(0x10000 * 0.0);
+	transform.matrix[2][2] = (XFixed)(0x10000 * 1.0);
+	XRenderSetPictureTransform(window.display, src_picture, &transform);
+	XRenderSetPictureFilter(window.display, src_picture, "fast", NULL, 0);
+	XRenderComposite(window.display, PictOpSrc, src_picture, None, dst_picture, 0, 0, 0, 0, 0, 0, 64,64);
 
-		if(src_picture != None)
-			XRenderFreePicture(window.display, src_picture);
-		if(dst_picture != None)
-			XRenderFreePicture(window.display, dst_picture);
-	}
+	if(src_picture != None)
+		XRenderFreePicture(window.display, src_picture);
+	if(dst_picture != None)
+		XRenderFreePicture(window.display, dst_picture);
 }
 
-static void rotate_into_image( Pixmap black_source, Pixmap white_source, float angle, s_image *image ){
-	// setup destination pixmap and mask
-	image->black_pixmap = XCreatePixmap(window.display, window.root_window, 64, 64, window.depth);
-	image->white_pixmap = XCreatePixmap(window.display, window.root_window, 64, 64, window.depth);
-	image->width = image->height = 64;
+static void rotate_into_image( unsigned init, Pixmap black_source, Pixmap white_source, float angle, s_image *image ){
+	if(init){
+		// setup destination pixmaps
+		image->black_pixmap = XCreatePixmap(window.display, window.root_window, 64, 64, window.depth);
+		image->white_pixmap = XCreatePixmap(window.display, window.root_window, 64, 64, window.depth);
+		image->width = image->height = 64;
+	}
 
 	_rotate_into_image(black_source, angle, image->black_pixmap);
 	_rotate_into_image(white_source, angle, image->white_pixmap);
 }
 
-static void generate_rotation_icons( ){
-	XftFont *font;
-	XGlyphInfo info;
-	XftResult result;
-	XftPattern *parsed, *matched;
-	XftDraw *surface;
+static XftFont *font;
+static XGlyphInfo info;
+
+static void draw_rotation_pixmap( Pixmap *pixmap, XColor *colors ){
+	// set color
 	XftColor xft_color;
+	xft_color.pixel = colors[ROTATION_COLOR].pixel;
+	xft_color.color.red = colors[ROTATION_COLOR].red;
+	xft_color.color.green = colors[ROTATION_COLOR].green;
+	xft_color.color.blue = colors[ROTATION_COLOR].blue;
+	xft_color.color.alpha = 0xffff;
+	// create pixmap with 'A'
+	*pixmap = XCreatePixmap(window.display, window.root_window, 64, 64, window.depth);
+		XSetForeground(window.display, window.gc, colors[SECONDARY_BG_COLOR].pixel);
+		XFillRectangle(window.display, *pixmap, window.gc, 0,0, 64,64);
+	XftDraw *surface = XftDrawCreate(window.display, *pixmap, window.visual, window.colormap);
+	XftDrawStringUtf8(surface, &xft_color, font, 32 - (info.width>>1), 32 + (info.height>>1) - (info.height - info.y), (const FcChar8 *)"A", 1);
+	XftDrawDestroy(surface);
+}
+
+static void generate_rotation_icons( unsigned init ){
 	Pixmap black_pixmap, white_pixmap;
 
-	// get font
-	parsed = XftNameParse("sans:medium:pixelsize=60");
-	matched = XftFontMatch(window.display, window.screen, parsed, &result);
-	font = XftFontOpenPattern(window.display, matched);
-	XftPatternDestroy(parsed);
-	XftPatternDestroy(matched);
-	XftTextExtentsUtf8(window.display, font, (const FcChar8 *)"A", 1, &info);
+	if(init){
+		// get font
+		XftPattern *parsed, *matched;
+		XftResult result;
+		parsed = XftNameParse("sans:medium:pixelsize=60");
+		matched = XftFontMatch(window.display, window.screen, parsed, &result);
+		font = XftFontOpenPattern(window.display, matched);
+		XftPatternDestroy(parsed);
+		XftPatternDestroy(matched);
+		XftTextExtentsUtf8(window.display, font, (const FcChar8 *)"A", 1, &info);
 
-	// black theme
-		// set color
-		xft_color.pixel = black_colors[ROTATION_COLOR].pixel;
-		xft_color.color.red = black_colors[ROTATION_COLOR].red;
-		xft_color.color.green = black_colors[ROTATION_COLOR].green;
-		xft_color.color.blue = black_colors[ROTATION_COLOR].blue;
-		xft_color.color.alpha = 0xffff;
-		// create pixmap with 'A'
-		black_pixmap = XCreatePixmap(window.display, window.root_window, 64, 64, window.depth);
-			XSetForeground(window.display, window.gc, black_colors[BG_COLOR].pixel);
-			XFillRectangle(window.display, black_pixmap, window.gc, 0,0, 64,64);
-		surface = XftDrawCreate(window.display, black_pixmap, window.visual, window.colormap);
-		XftDrawStringUtf8(surface, &xft_color, font, 32 - (info.width>>1), 32 + (info.height>>1) - (info.height - info.y), (const FcChar8 *)"A", 1);
-		XftDrawDestroy(surface);
+		// segfaults if font is loaded a second time
+	}
 
-	// white theme
-		// set color
-		xft_color.pixel = white_colors[ROTATION_COLOR].pixel;
-		xft_color.color.red = white_colors[ROTATION_COLOR].red;
-		xft_color.color.green = white_colors[ROTATION_COLOR].green;
-		xft_color.color.blue = white_colors[ROTATION_COLOR].blue;
-		xft_color.color.alpha = 0xffff;
-		// create pixmap with 'A'
-		white_pixmap = XCreatePixmap(window.display, window.root_window, 64, 64, window.depth);
-			XSetForeground(window.display, window.gc, white_colors[BG_COLOR].pixel);
-			XFillRectangle(window.display, white_pixmap, window.gc, 0,0, 64,64);
-		surface = XftDrawCreate(window.display, white_pixmap, window.visual, window.colormap);
-		XftDrawStringUtf8(surface, &xft_color, font, 32 - (info.width>>1), 32 + (info.height>>1) - (info.height - info.y), (const FcChar8 *)"A", 1);
-		XftDrawDestroy(surface);
+	// draw pixmap for each color set
+	draw_rotation_pixmap(&black_pixmap, black_colors);
+	draw_rotation_pixmap(&white_pixmap, white_colors);
 
-	// rotate pixmap into each icon
-	rotate_into_image(black_pixmap, white_pixmap,  1.0, &im_portrait); // top points left
-	rotate_into_image(black_pixmap, white_pixmap, -1.0, &im_portrait_inv); // top points right
-	rotate_into_image(black_pixmap, white_pixmap,  0.0, &im_landscape_inv); // top points down
+	// rotate pixmap into each orientation
+	rotate_into_image(init, black_pixmap, white_pixmap,  1.0, &im_portrait); // top points left
+	rotate_into_image(init, black_pixmap, white_pixmap, -1.0, &im_portrait_inv); // top points right
+	rotate_into_image(init, black_pixmap, white_pixmap,  0.0, &im_landscape_inv); // top points down
 
 	XFreePixmap(window.display, black_pixmap);
 	XFreePixmap(window.display, white_pixmap);
 }
 
+////////////////////////////////////////////////////////////////////////// THEME FILE
+
+const char *color_names[] = {
+	"bg_color",
+	"secondary_bg_color",
+	"line1_color",
+	"line2_color",
+	"fg1_color",
+	"fg2_color",
+	"bold_color",
+	"rotation_color",
+	"gray_color",
+	"yellow_color",
+	"green_color",
+	"red_color",
+	"blue_color",
+	"scrollbar_color",
+	"scroller_color",
+	"selection_color",
+	"secondary_selection_color",
+	"button_color",
+	"stepper_color"
+};
+
+static unsigned set_theme_color( XColor *color, char *value ){
+	unsigned length = strlen(value);
+	if(length != 4 && length != 7) return 0; // invalid
+	for(; length > 1; length--){
+		char c = value[length-1]|32;
+		if(!((c >= 'a' && c <= 'f') || (c >= '0' && c <= '9'))) return 0; // invalid
+	}
+	XParseColor(window.display, window.colormap, value, color);
+	XAllocColor(window.display, window.colormap, color);
+	return 1;
+}
+
+static void load_theme_file( unsigned load_black, unsigned load_white, char *theme ){
+	char filename[200], *p, *theme_name;
+	unsigned has_black = 0, has_white = 0;
+	FILE *in;
+	if(theme == NULL) return;
+	if(!strcmp(theme, "default.bw")){ debug_write("  %s is the default theme", theme); return; }
+
+	theme_name = p = &theme[strlen(theme)];
+
+	// get short name for user themes
+	if(*theme == '/') while(theme_name > theme && *(theme_name-1) != '/') theme_name--;
+
+	// get type of theme
+	while(p >= theme && *p != '.') p--;
+	if(!strcmp(p, ".bw")) has_black = has_white = 1;
+	else if(!strcmp(p, ".b")) has_black = 1;
+	else if(!strcmp(p, ".w")) has_white = 1;
+	else { debug_write("  %s theme file color set extension is missing", theme_name); return; } // invalid
+
+	if(!(load_black && has_black && load_white && has_white) && !(load_black && has_black) && !(load_white && has_white)){
+		debug_write("  %s theme file lacks requested color set [%s%s]", theme_name, (load_black ? "b" : ""), (load_white ? "w" : ""));
+		return; // missing color set
+	}
+
+	if(*theme != '/') snprintf(filename, sizeof(filename), "/etc/asui-themes/%s", theme);
+	else snprintf(filename, sizeof(filename), "%s", theme);
+	if((in = fopen(filename, "r")) == NULL){ debug_write("  %s theme file was not found", filename); return; } // missing file
+
+	debug_write("  load %s [%s%s]...", theme_name, (has_black && load_black ? "b" : ""), (has_white && load_white ? "w" : ""));
+	unsigned line_no = 0;
+	char black[8], white[8];
+	*black = *white = '#';
+	while(!feof(in)){
+		char line[1000], color[100];
+		int status, i;
+		fgets(line, sizeof(line), in);
+		if(feof(in)) break;
+		line_no++;
+
+		// ignore blank lines and comments
+		if(*line < 'a' || *line > 'z') continue;
+
+		if(has_black && has_white) status = sscanf(line, "%s = %s %s", color, &black[1], &white[1])-3;
+		else if(has_black) status = sscanf(line, "%s = %s", color, &black[1])-2;
+		else status = sscanf(line, "%s = %s", color, &white[1])-2;
+
+		if(status < 0){ debug_write("    %u: INVALID SYNTAX", line_no); continue; } // invalid
+		for(i = 0; i < NR_COLORS; i++)
+			if(!strcmp(color_names[i], color)) break;
+		if(i < NR_COLORS){
+			if(has_black && load_black){
+				if(set_theme_color(&black_colors[i], black)){
+					debug_write("    set %s:black = %s", color, black);
+				} else {
+					debug_write("    %u: INVALID COLOR %s:black = %s", line_no, color, black); // invalid
+				}
+			}
+			if(has_white && load_white){
+				if(set_theme_color(&white_colors[i], white)){
+					debug_write("    set %s:white = %s", color, white);
+				} else {
+					debug_write("    %u: INVALID COLOR %s:white = %s", line_no, color, white);
+				}
+			}
+		} else
+			debug_write("    %u: INVALID ELEMENT %s", line_no, color);
+	}
+	fclose(in);
+}
+
+static unsigned is_color_dark( XColor *color ){
+	if(color->red < 0x8000 && color->green < 0x8000 && color->blue < 0x8000) return 1;
+	return 0;
+}
+
 ////////////////////////////////////////////////////////////////////////// WINDOW INITIALIZATION
 
+static unsigned window_ready = 0;
+
+void window_reload( unsigned init ){
+	if(!window_ready) return;
+
+	// initialize colors
+	debug_write("loading default color theme...");
+	#define x_get_color(black, white) \
+		XParseColor(window.display, window.colormap, black, &black_colors[i]); XAllocColor(window.display, window.colormap, &black_colors[i  ]); \
+		XParseColor(window.display, window.colormap, white, &white_colors[i]); XAllocColor(window.display, window.colormap, &white_colors[i++]);
+	int i = 0;
+	// x_get_color("#black_color", "#white_color"); // color_name
+	#include "default-theme.inc"
+
+	// load theme file
+	if(!strcmp(cfg_theme_file_black, cfg_theme_file_white)){
+		debug_write("loading combined color theme file...");
+		load_theme_file(1, 1, cfg_theme_file_black);
+	} else {
+		debug_write("loading black color theme file...");
+		load_theme_file(1, 0, cfg_theme_file_black);
+		debug_write("loading white color theme file...");
+		load_theme_file(0, 1, cfg_theme_file_white);
+	}
+
+	// determine brightness of secondary_bg_color
+	black_is_dark = is_color_dark(&black_colors[SECONDARY_BG_COLOR]);
+	white_is_dark = is_color_dark(&white_colors[SECONDARY_BG_COLOR]);
+	theme_is_dark = (black_theme ? black_is_dark : white_is_dark);
+
+	// rotation icons
+	generate_rotation_icons(init);
+}
+
 static int window_create( unsigned first_time ){
 	if(first_time){
 		// open display
@@ -278,6 +429,8 @@ static int window_create( unsigned first_time ){
 			| (use_minimal_ui ? 0 : FocusChangeMask) | ExposureMask);
 		if(window.supports_rotation)
 			XRRSelectInput(window.display, window.window, RRScreenChangeNotifyMask);
+
+		window.x = window.y = 0;
 	}
 
 	// set name
@@ -307,33 +460,14 @@ static int window_create( unsigned first_time ){
 	}
 
 	if(first_time){
-		// initialize colors
-		#define x_get_color(black, white) \
-			XParseColor(window.display, window.colormap, black, &black_colors[i]); XAllocColor(window.display, window.colormap, &black_colors[i  ]); \
-			XParseColor(window.display, window.colormap, white, &white_colors[i]); XAllocColor(window.display, window.colormap, &white_colors[i++]);
-		int i = 0;
-		x_get_color("#000000", "#ffffff"); // bg_color
-		x_get_color("#444444", "#bbbbbb"); // line_color
-		x_get_color("#333333", "#e3e3e3"); // innerline_color
-		x_get_color("#dddddd", "#222222"); // fg1_color
-		x_get_color("#aaaaaa", "#555555"); // fg2_color
-		x_get_color("#ffffff", "#000000"); // white_color
-		x_get_color("#888888", "#777777"); // rotation_color
-		x_get_color("#777777", "#cccccc"); // gray_status_color
-		x_get_color("#cccc33", "#aaaa00"); // yellow_status_color
-		x_get_color("#33aa33", "#00aa00"); // green_status_color
-		x_get_color("#ff2222", "#ff0000"); // red_status_color
-		x_get_color("#aaaaff", "#6666dd"); // scrollbar_color
-		x_get_color("#555555", "#aaaaaa"); // scroller_color
-		x_get_color("#7777ff", "#0000dd"); // selection_color
-		x_get_color("#fb9200", "#ee8a00"); // secondary_selection_color
+		window_ready = 1;
+
+		// load theme colors and rotation icons
+		window_reload(1);
 
 		// load images
-		load_xpm(button_toflightmode_icon, &im_toflight);
-		load_xpm(button_tonormalmode_icon, &im_tonormal);
-		load_xpm(button_reboot_icon, &im_reboot);
-		load_xpm(button_shutdown_icon, &im_shutdown);
-		generate_rotation_icons();
+		load_themed_xpm(reboot_icon, reboot_white_icon, &im_reboot);
+		load_themed_xpm(shutdown_icon, shutdown_white_icon, &im_shutdown);
 		load_masked_xpm(volume_icon, &im_volume);
 		load_masked_xpm(brightness_icon, &im_brightness);
 
@@ -376,24 +510,29 @@ void window_set_theme( unsigned white_theme ){
 	hw_changes |= HW_THEME;
 
 	theme = (black_theme ? black_colors : white_colors);
+	theme_is_dark = (black_theme ? black_is_dark : white_is_dark);
 
 	bg_color = &theme[i++];
-	line_color = &theme[i++];
-	innerline_color = &theme[i++];
+	secondary_bg_color = &theme[i++];
+	line1_color = &theme[i++];
+	line2_color = &theme[i++];
 	fg1_color = &theme[i++];
 	fg2_color = &theme[i++];
-	white_color = &theme[i++];
+	bold_color = &theme[i++];
 	rotation_color = &theme[i++];
 
-	gray_status_color = &theme[i++];
-	yellow_status_color = &theme[i++];
-	green_status_color = &theme[i++];
-	red_status_color = &theme[i++];
+	gray_color = &theme[i++];
+	yellow_color = &theme[i++];
+	green_color = &theme[i++];
+	red_color = &theme[i++];
+	blue_color = &theme[i++];
 
 	scrollbar_color = &theme[i++];
 	scroller_color = &theme[i++];
 	selection_color = &theme[i++];
 	secondary_selection_color = &theme[i++];
+	button_color = &theme[i++];
+	stepper_color = &theme[i++];
 
 	assert(i == NR_COLORS);
 }
@@ -419,12 +558,19 @@ void window_auto_set_theme( unsigned initialize ){
 	window_set_theme(white_theme);
 }
 
-void window_map( ){
+void window_map( unsigned slide ){
 	debug_write("window_map()");
 
 	window_close_blank();
 	window_close_popup();
 
+	if(slide && cfg_window_slide_map){
+		if(window.width > window.height)
+			window_slide(0, window.height);
+		else
+			window_slide(-window.width, 0);
+	}
+
 	// map the window
 	{
 		XEvent e;
@@ -468,16 +614,48 @@ void window_map( ){
 
 	draw_window(1); // force
 
+	if(slide && cfg_window_slide_map){
+		if(window.width > window.height){
+			while(window.y > 0){
+				window_slide(0, window.y-SLIDE_STEP);
+				x_flush();
+				usleep(SLIDE_INTERVAL);
+			}
+		} else {
+			while(window.x < 0){
+				window_slide(window.x+SLIDE_STEP, 0);
+				x_flush();
+				usleep(SLIDE_INTERVAL);
+			}
+		}
+	}
+
 	// open popup window if it has been more than 10 seconds since it was last closed
 	if(current_dialog_draw != draw_alarm && cfg_popup_onmap && current_time - popup_window_close_time > 10.0)
 		window_open_popup();
 }
 
-void window_unmap( ){
+void window_unmap( unsigned slide ){
 	debug_write("window_unmap()");
 
 	window_close_popup();
 
+	if(slide && cfg_window_slide_unmap){
+		if(window.width > window.height){
+			while(window.y < (int)window.height-SLIDE_STEP){
+				window_slide(0, window.y+SLIDE_STEP);
+				x_flush();
+				usleep(SLIDE_INTERVAL);
+			}
+		} else {
+			while(window.x > -(int)window.width+SLIDE_STEP){
+				window_slide(window.x-SLIDE_STEP, 0);
+				x_flush();
+				usleep(SLIDE_INTERVAL);
+			}
+		}
+	}
+
 	// close dialog if open -- this is needed to reset anything it did to the main loop (the process viewer refresh)
 	dialog_reset();
 
@@ -495,12 +673,22 @@ void window_unmap( ){
 	XUnmapWindow(window.display, window.window);
 	x_flush();
 
+	// move window back to original position
+	if(window.x || window.y) window_slide(0,0);
+
 	window.is_mapped = 0;
 }
 
+void window_slide( int x, int y ){
+	XWindowChanges values;
+	window.x = values.x = x;
+	window.y = values.y = y;
+	XConfigureWindow(window.display, window.window, CWX|CWY, &values);
+}
+
 void window_close( unsigned terminate ){
 	if(window.is_mapped)
-		window_unmap();
+		window_unmap(0);
 
 	if(terminate)
 		text_free_fonts();
@@ -622,7 +810,7 @@ void rotate_screen( e_orientation new_orientation, unsigned keep_mapped ){
 
 			x_set_color(bg_color);
 			x_fill_rectangle(0, 0, size,size);
-			draw_text("rotating...", 32, white_color, 240, 240, ALIGN_CENTER, ALIGN_MIDDLE, 0);
+			draw_text("rotating...", 32, fg1_color, 240, 240, ALIGN_CENTER, ALIGN_MIDDLE, 0);
 			XFreeGC(window.display, gc);
 			x_flush();
 
@@ -659,7 +847,7 @@ void rotate_screen( e_orientation new_orientation, unsigned keep_mapped ){
 		if(keep_mapped){
 			XUnmapWindow(window.display, window.window);
 			x_flush();
-		} else window_unmap();
+		} else window_unmap(0);
 		// resize window
 		XWindowChanges changes;
 		changes.width = window.width;
@@ -783,7 +971,7 @@ void update_orientation( XEvent *event ){
 		}
 
 //		if(was_mapped)
-//			window_map();
+//			window_map(0);
 
 		// close the popup window
 		window_close_popup();
@@ -1039,7 +1227,7 @@ void draw_popup_window( ){
 		draw_clock_widget(1, cx-(hw_device_locked ? 80 : 200), cy+70);
 	} else {
 		// border
-		x_set_color(line_color);
+		x_set_color(line1_color);
 		x_fill_rectangle(0, 0, w,h);
 		// background
 		x_set_color(bg_color);
@@ -1049,7 +1237,7 @@ void draw_popup_window( ){
 		use_minimal_ui = 1;
 		draw_clock_widget(1, (hw_device_locked ? 125 : 5), (hw_device_locked ? 100 : 155));
 	}
-	draw_text("tap to close", 14, line_color, w>>1, (cfg_popup_fullscreen ? h : h-5), ALIGN_CENTER, ALIGN_BOTTOM, 0);
+	draw_text("tap to close", 14, line1_color, w>>1, (cfg_popup_fullscreen ? h : h-5), ALIGN_CENTER, ALIGN_BOTTOM, 0);
 	drawing_popup_window = use_minimal_ui = 0;
 
 	// restore state
